//
//    Copyright 2011 Alexey Shutov
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "ChannelsSetupForm.h"

#include <QCheckBox>
#include <QDebug>
#include <QPushButton>
#include <QSpinBox>

#include "DeviceChTree/DeviceChModel.h"
#include "lib-common/BaseDeviceAppConfig.h"
#include "ui_ChannelsSetupForm.h"

namespace  {
const int THRESHOLD_SINGLE_STEP = 16;
const int BASELINE_SINGLE_STEP = 1;
}

ChannelsSetupForm::ChannelsSetupForm(BaseDeviceAppConfig *config, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::ChannelsSetupForm)
    , config(config)
    , tableWidgetChangeEnable(false)
    , treeModel(new DeviceChModel(this))
{
    ui->setupUi(this);

    tableWidgetChangeEnable = true;

    ui->deviceTreeView->setTreeModel(treeModel);
}

ChannelsSetupForm::~ChannelsSetupForm()
{
    delete ui;
}

DeviceChTreeView *ChannelsSetupForm::getDeviceChTreeView() const
{
    return ui->deviceTreeView;
}

void ChannelsSetupForm::setControlStateEnable(bool enable)
{
    treeModel->setEnabled(enable);
}

void ChannelsSetupForm::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
