/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADC64CORE_H
#define ADC64CORE_H

#include "lib-common/ReadoutAppCore.h"

#include <QMap>

#include "fsm/FsmState.h"
#include "adc64-common/AdcStatus.h"

#include "lib-common/BaseDeviceAppConfig.h"

class ReadoutAppConfigDb;
class DominoDeviceManager;
class Adc64SystemFsm;
class RemoteControlServerStatus;
struct DeserializersControlBuildParams;

class Adc64Core : public ReadoutAppCore
{
    Q_OBJECT
public:
    explicit Adc64Core(QObject *parent = nullptr);

signals:
    void configChanged(const BaseDeviceAppConfig &) const;
    void startCalibration(const ConfigKey &) const;
    void startAdcSrdDesCalibration(const ConfigKey &calKey, bool hardMode,
                                   const QString &resDir) const;

public slots:
    void setConfig(const ConfigKey &key, const BaseDeviceAppConfig &newConfig);
    void setDeserializersControlBuildParams(DeviceIndex index, const DeserializersControlBuildParams &bp);
    void runError(const QString &text = {});
    void performLocalStart();
    void performLocalStop();
    void performCalibrate();
    void onFsmStateChange(const FsmState &state, const FsmState &oldState) override;
    void onRemoteStartRequest(const RcRunId &runId, const QVariant &start_event_number);
    void onRemoteStopRequest(const RcRunId &runId, const QVariant &stop_event_number);
    virtual void remoteStatusUpdated(const RemoteControlServerStatus &status) override;
    void requestAdcSerDesCalibration(bool hardMode, const QString &dir);
    void setAdcDeviceStatus(const AdcStatus &status);

protected:
    const BaseFsm *getFsm() const override;
    BaseDeviceAppConfig getBaseAppConfig() const final { return config; }

private:
    void sendRunDocument();
    void refreshProgramDescription();

    ReadoutAppConfigDb * const configDb;
    DominoDeviceManager * const deviceMgr;
    Adc64SystemFsm * const fsm;

    BaseDeviceAppConfig config;
    ConfigKey config_key;
    ConfigKey calKey;
    bool adcSerDesCalHardMode;
    QString adcSerDesCalResDir;
};

#endif // ADC64CORE_H
