//
//    Copyright 2011 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "DominoDeviceManager.h"

#include <QDebug>
#include <QMetaType>

#include "Adc64SignalRouter.h"
#include "adc64-common/AdcDeviceProxy.h"
#include "adc64-common/DominoDevice.h"
#include "base/DeviceIndexEnabled.h"
#include "lib-common/BaseDeviceAppConfig.h"


DominoDeviceManager::DominoDeviceManager(QObject *parent)
    : QObject(parent)
{
    qRegisterMetaType<AdcStatus>();
}

DominoDeviceManager::~DominoDeviceManager()
{
}

void DominoDeviceManager::setHostAddress(const DeviceIndex &index, const QHostAddress &address, int ttl)
{
    if (! devices.contains(index))
        return;
    devices[index]->setHostAddress(index, address, ttl);
}

void DominoDeviceManager::setDevices(const DeviceIndexEnabledMap &deviceList)
{
    const DeviceIndexSet &newDeviceSet = deviceList.getEnabled();
    const DeviceIndexSet &curSet = makeDeviceIndexSet(devices.keys());
    if(curSet == newDeviceSet)
        return;

    const auto &toDel = curSet-newDeviceSet;

    const auto &toAdd = newDeviceSet-curSet;
    for(const auto &index : toAdd) {
        DominoDevice * const dev = new DominoDevice(index);

        connect(dev, &DominoDevice::adcStatusUpdated, this, &DominoDeviceManager::adcStatusUpdated);
        connect(dev, &ModularMregDevice::deserializersControlBpUpdated, this, &DominoDeviceManager::deserializersControlBpUpdated);

        auto *deviceProxy = new AdcDeviceProxy(dev, this);
        devices.insert(index, QSharedPointer<AdcDeviceProxy>(deviceProxy));

        Adc64SignalRouter::instance().connect_DeviceGlobal(deviceProxy, dev);
    }

    removeDevices(toDel);
    //    qDebug() << "DominoDeviceManager::setHostAddressList:" << devices.size() << "devices created";
}

void DominoDeviceManager::removeDevices(const DeviceIndexSet &setToDel)
{
    for(const auto &i : setToDel) {
        devices.remove(i);
    }
}
