#ifndef TRCCONFIGDISPATCHER_H
#define TRCCONFIGDISPATCHER_H

#include "lib-common/ReadoutAppConfigDispatcher.h"

class DeviceIndexEnabledMap;
class PhysicalTriggerConfig;
class RcTrigConfig;
class SpillConfig;

class TrcConfigDispatcher : public ReadoutAppConfigDispatcher
{
    Q_OBJECT
    explicit TrcConfigDispatcher(QObject *parent = nullptr);
    Q_DISABLE_COPY_MOVE(TrcConfigDispatcher)
public:
    static TrcConfigDispatcher & instance()
    {
        static TrcConfigDispatcher obj;
        return obj;
    }

public slots:
    void deviceInfoUpdated(const DeviceIndex &index, quint16 auxNch, quint16 phyNch);

public:
    template<class T>
    void subscribeSpillConfig(T *obj) {
            connect(this, &TrcConfigDispatcher::spillCfgChanged,
                    obj, &T::setSpillCfg,
                    Qt::UniqueConnection);

            connect(obj, &T::spillCfgChanged,
                    this, &TrcConfigDispatcher::setSpillCfg,
                    Qt::UniqueConnection);
            logConnection(obj, "SpillConfig");
    }
signals:
    void spillCfgChanged(const SpillConfig &) const;
public slots:
    void setSpillCfg(const SpillConfig &cfg);

public:
    template<class T>
    void subscribeRcTrigConfig(T *obj) {
            connect(this, &TrcConfigDispatcher::rcTrigCfgChanged,
                    obj, &T::setRcTrigCfg,
                    Qt::UniqueConnection);

            connect(obj, &T::rcTrigCfgChanged,
                    this, &TrcConfigDispatcher::setRcTrigCfg,
                    Qt::UniqueConnection);
            logConnection(obj, "RcTrigConfig");
    }
signals:
    void rcTrigCfgChanged(const RcTrigConfig &) const;
public slots:
    void setRcTrigCfg(const RcTrigConfig &cfg);

public:
    template<class T>
    void subscribePhysicalTriggerConfig(T *obj) {
            connect(this, &TrcConfigDispatcher::phyTrigCfgChanged,
                    obj, &T::setPhyTrigCfg,
                    Qt::UniqueConnection);

            connect(obj, &T::phyTrigCfgChanged,
                    this, &TrcConfigDispatcher::setPhyTrigCfg,
                    Qt::UniqueConnection);
            logConnection(obj, "PhysicalTriggerConfig");
    }
signals:
    void phyTrigCfgChanged(const PhysicalTriggerConfig &) const;
public slots:
    void setPhyTrigCfg(const PhysicalTriggerConfig &cfg);

private:
    void sendConfigKeyCache() override;
};
#endif // TRCCONFIGDISPATCHER_H
