﻿#include "BmnTrcDeviceChTableView.h"

#include <QHeaderView>

#include "BmnTrcTreeColumns.h"
#include "BmnTrcSpinBoxDelegate.h"

namespace {
const static QVector<TrcTreeColumns> DELEGATE_COLS = {
    TrcTreeColumns::COL_DELAY,
    TrcTreeColumns::COL_REDUC_FACTOR,
    TrcTreeColumns::COL_AFTER_TIME,
    TrcTreeColumns::COL_BEFORE_TIME,
};
}

BmnTrcDeviceChTableView::BmnTrcDeviceChTableView(QWidget *parent)
    : QTableView(parent)
    , delegate(new BmnTrcSpinBoxDelegate(this))
{
    horizontalHeader()->setStretchLastSection(false);
    horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

void BmnTrcDeviceChTableView::setModel(QAbstractItemModel *model)
{
    QTableView::setModel(model);
    for(auto c : DELEGATE_COLS)
        setItemDelegateForColumn(static_cast<int>(c), delegate);

    connect(model, &QAbstractItemModel::modelReset, this, &BmnTrcDeviceChTableView::onModelReset);
    addDelegates();
}

void BmnTrcDeviceChTableView::onModelReset()
{
    addDelegates();
}

void BmnTrcDeviceChTableView::addDelegates()
{
    const auto &m = model();
    int rows = m->rowCount();
    for(int r=0; r<rows; ++r) {
        for(auto c : DELEGATE_COLS) {
            auto i = m->index(r, static_cast<int>(c));
            if(!isPersistentEditorOpen(i))
                openPersistentEditor(i);
        }
    }
}
