#include "CruFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.cru")
}

CruFsm::CruFsm(QObject *parent)
    : MStreamDeviceMgrFsm{parent}
{
    setObjectName(__func__);
    recreateInput(new CruFsmInput());
}

bool CruFsm::isTransitionAllowed(const FsmState &toState) const
{
    switch (getState()) {
    case FsmState::Configure:
        if(toState == FsmState::Starting)
            return allDevicesWritten()
                // && allMstreamReseted()
                && allDevPreRunReseted();
        break;
    default:;
    }

    return MStreamDeviceMgrFsm::isTransitionAllowed(toState);
}

void CruFsm::setRunId(RcRunId runId)
{
    if(getInput()->runId == runId)
        return;
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << runId;
    getInput()->runId = runId;
    evaluate();
}

void CruFsm::setDevRunId(const DeviceIndex &index, RcRunId runId)
{
    auto &map= getInput()->devRunId;
    auto it = map.find(index);
    if(it == map.end())
        map[index] = runId;
    else if(*it != runId)
        *it = runId;
    else
        return;

    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << index << runId;
    evaluate();
}

void CruFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

qint64 CruFsm::stateTimeoutMs(const FsmState &state) const
{
    switch (state) {
    case FsmState::Configure:
        return 1e3;
    default:
        return DeviceMgrFsm::stateTimeoutMs(state);
    }
}

bool CruFsm::allDevPreRunReseted() const
{
    auto i = getInput();
    for(auto key : i->devSet) {
        auto it = i->devRunId.constFind(key);
        if(it==i->devRunId.cend() || *it != i->runId)
            return false;
    }
    return true;
}

QDebug operator<<(QDebug dbg, const CruFsmInput &in)
{
    dbg << MStreamDeviceMgrFsmInput(in);

    dbg << "runId:" << in.runId << "; ";
    dbg << "devRunId:" << in.devRunId << "; ";

    return dbg;
}
