#include "FileWatcherConfigDispatcher.h"

#include <optional>
#include <QDebug>

#include "base/ConfigIndexName.h"
#include "base/util.h"

#include "FileWatcherConfig.h"

class FileWatcherConfigDispatcherPrivate
{
public:
    std::optional<ConfigIndexName> cacheCIN;
    FileWatcherConfig cfg;
};

FileWatcherConfigDispatcher::FileWatcherConfigDispatcher(QObject *parent)
    : AbstractDispatcher(parent),
    d_ptr(new FileWatcherConfigDispatcherPrivate)
{
    qRegisterMetaType<ConfigIndexName>();
    qRegisterMetaType<FileWatcherConfig>();
}

FileWatcherConfigDispatcher::~FileWatcherConfigDispatcher()
{
    delete d_ptr;
}

void FileWatcherConfigDispatcher::sendCache()
{
    Q_D(const FileWatcherConfigDispatcher);
    if (!d->cacheCIN.has_value())
        return;
    emit switch_to_config(d->cacheCIN.value());
    emit allowRemoteControlChanged(d->cfg.allowRemoteControl);
    emit configChanged(d->cfg);
}

void FileWatcherConfigDispatcher::configSelectorChanged(const ConfigIndexName &cin)
{
    Q_D(FileWatcherConfigDispatcher);

    if (d->cacheCIN.has_value() && d->cacheCIN.value() == cin)
        return;
    d->cacheCIN = cin;
    emit switch_to_config(cin);
}

void FileWatcherConfigDispatcher::setAllowRemoteControl(bool allow)
{
    Q_D(FileWatcherConfigDispatcher);
    if(d->cfg.allowRemoteControl == allow)
        return;
    d->cfg.allowRemoteControl = allow;

    sendCache();
}

void FileWatcherConfigDispatcher::setConfig(const FileWatcherConfig &newConfig)
{
    if (sender())
        qDebug() << "app config changed by" << qObjectAncestryStr(sender());
    Q_D(FileWatcherConfigDispatcher);
    if (d->cfg == newConfig)
        return;

    d->cfg = newConfig;
    sendCache();
}
