#include "FileWatcherSignalRouter.h"

#include <QCoreApplication>

#include "FileWatcherConfigDb.h"
#include "FileWatcherConfigDispatcher.h"
#include "FileWatcherCore.h"
#include "FileWatcherFsm.h"
//#include "FileWatcherMainWindow.h"
#include "ConsoleReader.h"
#include "DataSender.h"
#include "FileReader.h"

#include "configuration-manager/ConfigurationManager.h"
#include "pnp-server/PnpDiscovery.h"
#include "remote-control-server/RemoteControlServer.h"

FileWatcherSignalRouter::FileWatcherSignalRouter()
    : BaseSignalRouter()
{
    setConfigDispatcher(&FileWatcherConfigDispatcher::instance());
}

void FileWatcherSignalRouter::setCore(FileWatcherCore *p)
{
    setBaseAppCore(p);
    core = p;
}

void FileWatcherSignalRouter::setFsm(FileWatcherFsm *p)
{
    fsm = p;
    setMainFsm(fsm);
}

void FileWatcherSignalRouter::connectAll() const
{
    BaseSignalRouter::connectAll();

    connect_ConfigurationDispatcher();
    connect_ConsoleReader();

    if(core && fileMonitor)
        connect(core, &FileWatcherCore::enableFileMonitor,
            fileMonitor, &FileReader::setEnabled);

    if(dataSender && fileMonitor)
        connect(fileMonitor, &FileReader::fileDataUpdate,
                dataSender, &DataSender::fileDataUpdate);

    if(core && dataSender)
        connect(dataSender, &DataSender::stat_updated,
                core, &FileWatcherCore::mStreamStatUpdated);
}

void FileWatcherSignalRouter::connect_ConfigurationDispatcher() const
{
    if (!configDispatcher)
        return;

    if(configDb) {
        configDispatcher->subscribeCS(configDb);
        configDispatcher->subscribeConfig(configDb);
    }

    if (core) {
        configDispatcher->subscribeCS(core);
        configDispatcher->subscribeConfig(core);
    }

//    if (mainWindow) {
//        configDispatcher->subscribeCS(mainWindow);
//        configDispatcher->subscribeAllowRemoteControl(mainWindow);
//    }

    if(fileMonitor) {
        configDispatcher->subscribeConfig(fileMonitor);
    }

    if(remoteControlServer) {
        configDispatcher->subscribeAllowRemoteControl(remoteControlServer);
    }

    if (configurationManager)
        configDispatcher->subscribeCS(configurationManager);
}

void FileWatcherSignalRouter::connect_PnpServer() const
{
    BaseSignalRouter::connect_PnpServer();
    if(dataSender)
        PnpDiscovery::subscribeMyProgram(dataSender);
}

void FileWatcherSignalRouter::connect_RemoteControlServer() const
{
    BaseSignalRouter::connect_RemoteControlServer();
    if(core) {
        //todo: move to base
        connect(core, &FileWatcherCore::sendRemoteState,
                remoteControlServer, &RemoteControlServer::sendState);

        connect(remoteControlServer, &RemoteControlServer::startRequest,
                core, &FileWatcherCore::remoteStartRequest);
        connect(core, &FileWatcherCore::sendRemoteState,
                remoteControlServer, &RemoteControlServer::sendState);
        connect(remoteControlServer, &RemoteControlServer::stopRequest,
                core, &FileWatcherCore::remoteStopRequest);
        connect(core, &FileWatcherCore::sendRunError,
                remoteControlServer, &RemoteControlServer::onRunError);
        connect(core, &FileWatcherCore::sendRunWarning,
                remoteControlServer, &RemoteControlServer::onRunWarning);
    }
//    if(mainWindow)
//        connect(remoteControlServer, &RemoteControlServer::connectionStatusUpdated,
//                mainWindow, &FileWatcherMainWindow::remoteStatusUpdated);
}

void FileWatcherSignalRouter::connect_FSM() const
{
    BaseSignalRouter::connect_FSM();

    if(!fsm)
        return;

    if(fileMonitor) {
        connect(fileMonitor, &FileReader::fileValidUpdate,
                fsm, &FileWatcherFsm::setFileValid);
    }

//    if(mainWindow) {
//        connect(mainWindow, &FileWatcherMainWindow::printFsmInput,
//                mainFsm, &BaseFsm::printFsmInput);

//        connect(mainFsm, &BaseFsm::stateChanged,
//                mainWindow, &FileWatcherMainWindow::onFsmStateChange);
    //    }
}

void FileWatcherSignalRouter::connect_ConsoleReader() const
{
    QObject::connect(consoleReader, &ConsoleReader::quit,
                     QCoreApplication::instance(), &QCoreApplication::quit);

    connect(consoleReader, &ConsoleReader::printStat,
                     core, &FileWatcherCore::printStat);
    connect(consoleReader, &ConsoleReader::toggleStatPrint,
                     core, &FileWatcherCore::toggleStatPrint);
    connect(consoleReader, &ConsoleReader::setFilePath,
                     core, &FileWatcherCore::setFilePath);
}
