#ifndef HRBCONFIGDISPATCHER_H
#define HRBCONFIGDISPATCHER_H

#include <boost/shared_ptr.hpp>

#include "config/HrbAppConfig.h"
#include "device-modules/ModularDeviceConfig.h"
#include "lib-common/BaseConfigDispatcher.h"

class DeviceIndexEnabledMap;

class HrbConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
    explicit HrbConfigDispatcher(QObject *parent = nullptr);
public:
    static HrbConfigDispatcher & instance()
    {
        static HrbConfigDispatcher obj;
        return obj;
    }

    // ===== HrbAppConfig begin =====
public:
    template<class T>
    void subscribeConfig(T *obj) {
        connect(this, &HrbConfigDispatcher::configChanged,
                obj, &T::setConfig,
                Qt::UniqueConnection);

        connect(obj, &T::configChanged,
                this, &HrbConfigDispatcher::setConfig,
                Qt::UniqueConnection);
        logConnection(obj, "HrbAppConfig");
        sendCache();
    }
signals:
    void configChanged(ConfigKey key, const HrbAppConfig &) const;
public slots:
    void setConfig(const HrbAppConfig &newConfig);
    // ===== HrbAppConfig end  =====
    // ===== HrbConfig begin =====
public:
    template<class T>
    void subscribeHrbDevice(T *obj) {
        connect(this, &HrbConfigDispatcher::hrbDeviceConfigChanged,
                obj, &T::setDeviceConfig,
                Qt::UniqueConnection);
        logConnection(obj, "HrbConfig");
        sendHrbConfigCache();
    }
    template<class T>
    void subscribeHrbModel(T *obj) {
//        connect(this, &HrbConfigDispatcher::hrbDeviceConfigChanged,
//                obj, &T::setDeviceConfig,
//                Qt::UniqueConnection);
        connect(obj, &T::hrbDeviceConfigChanged,
                this, &HrbConfigDispatcher::setHrbDeviceConfig,
                Qt::UniqueConnection);
        logConnection(obj, "HrbConfig");
//        sendHrbConfigCache();
    }
signals:
    void hrbDeviceConfigChanged(ConfigKey key, const ModularDeviceConfig &) const;
public slots:
    void setHrbDeviceConfig(const ModularDeviceConfig &cfg);
    // ===== HrbConfig end  =====
    // ===== TtbConfig begin =====
public:
    template<class T>
    void subscribeTtbDevice(T *obj) {
        connect(this, &HrbConfigDispatcher::ttbDeviceConfigChanged,
                obj, &T::setDeviceConfig,
                Qt::UniqueConnection);
        logConnection(obj,  "TtbConfig");
        sendTtbConfigCache();
    }
    template<class T>
    void subscribeTtbModel(T *obj) {
//        connect(this, &HrbConfigDispatcher::ttbDeviceConfigChanged,
//                obj, &T::setDeviceConfig,
//                Qt::UniqueConnection);
//        sendTtbConfigCache();
        connect(obj, &T::ttbDeviceConfigChanged,
                this, &HrbConfigDispatcher::setTtbDeviceConfig,
                Qt::UniqueConnection);
        logConnection(obj, "TtbConfig");
    }
signals:
    void ttbDeviceConfigChanged(ConfigKey key, const ModularDeviceConfig &) const;
public slots:
    void setTtbDeviceConfig(const ModularDeviceConfig &cfg);
    // ===== TtbConfig end  =====
    // ===== HrbTriggerSetup begin =====
public:
    template<class T>
    void subscribeHrbTriggerSetup(T *obj) {
        connect(this, &HrbConfigDispatcher::hrbTriggerSetupChanged,
                obj, &T::setTriggerSetup,
                Qt::UniqueConnection);
        connect(obj, &T::hrbTriggerSetupChanged,
                this, &HrbConfigDispatcher::setTriggerSetup,
                Qt::UniqueConnection);
        logConnection(obj, "HrbTriggerSetup");
        emit hrbTriggerSetupChanged(config_key, cache->triggerSetup);
    }
signals:
    void hrbTriggerSetupChanged(ConfigKey key, const HrbTriggerSetup &) const;
public slots:
    void setTriggerSetup(const HrbTriggerSetup &cfg);
    // ===== HrbTriggerSetup end  =====


public slots:
    void setDeviceList(const DeviceIndexEnabledMap &map) override;

protected:
    DeviceIndexEnabledMap getDeviceIndexEnabledMap() const override;

private:
    void sendConfigKeyCache() override {
        emit configKeyChanged(config_key);

        emit configChanged(config_key, *cache);
        emit hrbTriggerSetupChanged(config_key, cache->triggerSetup);
        sendHrbConfigCache();
        sendTtbConfigCache();
    }
    void sendHrbConfigCache() const {
        for(auto cfg : cache->hrb.values())
            emit hrbDeviceConfigChanged(config_key, cfg);
    }
    void sendTtbConfigCache() const {
        for(auto cfg : cache->ttb.values())
            emit ttbDeviceConfigChanged(config_key, cfg);
    }

    boost::shared_ptr<HrbAppConfig> cache;
};
#endif // HRBCONFIGDISPATCHER_H
