#ifndef EVBGENCONFIGDISPATCHER_H
#define EVBGENCONFIGDISPATCHER_H

#include "base/AbstractDispatcher.h"

class EvbGenConfigDispatcherPrivate;
class ConfigIndexName;
struct EvbGenConfig;
struct GenServerConfig;

class EvbGenConfigDispatcher : public AbstractDispatcher
{
    Q_OBJECT
    Q_DISABLE_COPY_MOVE(EvbGenConfigDispatcher)
    explicit EvbGenConfigDispatcher(QObject *parent = nullptr);

public:
    // Singleton
    static EvbGenConfigDispatcher & instance()
    {
        // thread-safe since C++11
        static EvbGenConfigDispatcher obj;
        return obj;
    }

    virtual ~EvbGenConfigDispatcher() override;

    virtual void sendCache();

    template<class T>
    void subscribeEvbGenConfig(T *obj) {
        connect(obj, &T::evbGenConfigChanged,
                this, &EvbGenConfigDispatcher::setEvbGenConfig);
        connect(this, &EvbGenConfigDispatcher::evbGenConfigChanged,
                obj, &T::setEvbGenConfig);
        logConnection(obj, "EvbGenConfig");
        sendCache();
    }

    template<class T>
    void subscribeEvbGenServerConfig(T *obj) {
        connect(obj, &T::evbGenServerConfigChanged,
                this, &EvbGenConfigDispatcher::setEvbGenServerConfig);
        connect(this, &EvbGenConfigDispatcher::evbGenServerConfigChanged,
                obj, &T::setEvbGenServerConfig);
        logConnection(obj, "EvbGenServerConfig");
        sendCache();
    }

    template<class T>
    void subscribeCS(T *obj) {
        connect(this, &EvbGenConfigDispatcher::switch_to_config,
                obj, &T::configSelectorChanged,
                Qt::UniqueConnection);

        connect(obj, &T::switch_to_config,
                this, &EvbGenConfigDispatcher::configSelectorChanged,
                Qt::UniqueConnection);
        logConnection(obj, "ConfigIndexName");
    }

signals:
    void switch_to_config(const ConfigIndexName &conf) const;
    void evbGenConfigChanged(const EvbGenConfig &) const;
    void evbGenServerConfigChanged(const GenServerConfig &) const;

public slots:
    void configSelectorChanged(const ConfigIndexName &cin);
    virtual void setEvbGenConfig(const EvbGenConfig &newConfig);
    virtual void setEvbGenServerConfig(const GenServerConfig &newConfig);

private:
    Q_DECLARE_PRIVATE(EvbGenConfigDispatcher)
    EvbGenConfigDispatcherPrivate * const d_ptr;
};

#endif // EVBGENCONFIGDISPATCHER_H
