#include "EvbGenFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evbGen")
}

EvbGenFsm::EvbGenFsm(QObject *parent)
    : BaseFsm{parent}
{
    setObjectName(__func__);
    recreateInput(new EvbGenFsmInput());
}

bool EvbGenFsm::isTransitionAllowed(const FsmState &toState) const
{
    switch (getState()) {
    case FsmState::Ready:
        if(toState == FsmState::Starting) return true;
        break;
    case FsmState::Starting:
        if(toState == FsmState::Run) return allInState(FsmState::Run);
        break;
    case FsmState::Run:
        if(toState == FsmState::Stopping) return true;
        break;
    case FsmState::Stopping:
        if(toState == FsmState::Ready) return allInState(FsmState::Ready);
        break;
    default:;
    }
    return BaseFsm::isTransitionAllowed(toState);
}

void EvbGenFsm::setGenNum(int genNum)
{
    auto i = getInput();
    if(i->genNum == genNum)
        return;
    i->genNum = genNum;
    evaluate();
}

void EvbGenFsm::setGenState(int id, FsmState st)
{
    qInfo() << __PRETTY_FUNCTION__ << id << st;
    auto i = getInput();
    auto it = i->genState.find(id);
    if(it != i->genState.end() && *it == st)
        return;
    i->genState[id] = st;
    evaluate();
}

bool EvbGenFsm::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    if(getState() == FsmState::Error) {
        if (allInState(FsmState::Ready)) {
            setState(FsmState::Ready);
            return true;
        }
    } else {
        if(hasAnyErr()) {
            qDebug(fsm) << __PRETTY_FUNCTION__ << "Go to Error state (has some err)";
            setState(FsmState::Error);
            return true;
        }
    }

    return false;
}

void EvbGenFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

bool EvbGenFsm::hasAnyErr() const
{
    auto i = getInput();
    for(auto j=0; j< i->genNum; ++j) {
        if(i->genState.value(j, FsmState::Error) == FsmState::Error)
            return true;
    }
    return false;
}

bool EvbGenFsm::allInState(FsmState st) const
{
    auto i = getInput();
    for(auto j=0; j< i->genNum; ++j) {
        if(i->genState.value(j, FsmState::Unknown) != st)
            return false;
    }
    return true;
}

QDebug operator<<(QDebug dbg, const EvbGenFsmInput &in)
{
    dbg << BaseFsmInput(in);

    dbg << "genNum:" << in.genNum << "; "
        << "genState:" << in.genState << "; ";
    return dbg;
}
