#include "BaseEvbClientFsm.h"
#include "BaseEvbClientFsm.h"

#include "ClientFsm.h"

#include <QDebug>
#include <QLoggingCategory>

#include "base/ClientIndex.h"

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evb.BaseEvbClientFsm")

const QMap<FsmState, FsmState> STATE_WORK_LOOP{
    {FsmState::Ready, FsmState::Configure},
    {FsmState::Configure, FsmState::Run},
    {FsmState::Run, FsmState::Ready},
};
}

BaseEvbClientFsm::BaseEvbClientFsm(QObject *parent)
    : BaseFsm(parent)
{
    setObjectName(__func__);
}

bool BaseEvbClientFsm::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    if(getState() == FsmState::Error) {
        if (!hasAnyError() && checkSubStates(FsmState::Ready)) {
            setState(FsmState::Ready);
            return true;
        }
    } else {
        if(hasAnyError()) {
            setState(FsmState::Error);
            return true;
        }
        const auto loopIt = STATE_WORK_LOOP.constFind(getState());
        if(loopIt != STATE_WORK_LOOP.cend()) {
            const auto &reqSt = getRequestState();
            if(reqSt == loopIt.value()) {
                doReqStateActions(reqSt);
                if(checkSubStates(reqSt)) {
                    acceptRequestedState();
                    return true;
                }
            } else {
                if(!checkSubStates(getState())) {
                    setState(FsmState::Error);
                    return true;
                }
            }
        }
    }

    return false;
}

qint64 BaseEvbClientFsm::stateTimeoutMs(const FsmState &state) const
{
    return BaseFsm::stateTimeoutMs(state);
}

qint64 BaseEvbClientFsm::requestTimeoutMs() const
{
    if(getState() == FsmState::Error)
        return -1;

    switch (getRequestState()) {
    case FsmState::Configure:
        return 10000;
    case FsmState::Run:
    case FsmState::Ready:
        return 1000;
    default:;
    }
    return BaseFsm::requestTimeoutMs();
}
