#ifndef EVBMAINWINDOW_H
#define EVBMAINWINDOW_H

#include "Combiner.h"
#include "EvBConfigDb.h"
#include "RunControl.h"

#include "base/app/BaseMainWindow.h"
#include "base/RcRunId.h"
#include "base/ClientIndexEnabled.h"
#include "fsm/FsmState.h"
#include "configuration-manager/ConfigurationManager.h"
#include "pnp-server/PNPMessage.h"
#include "EventBuilderFsm.h"
#include "TcpClientStatus.h"

#include <QMainWindow>
#include <QMap>

namespace Ui {
class EvbMainWindow;
}

class Analizer;
class Client;
class ConfigSelector;
class EvbClientStatusTableView;
class EvBConfigDispatcher;
class EvbFileNameDialog;
struct EvBStartupOptions;
class AdjustBufferDialog;
class AdjustBufferModel;
class FileSplitDialog;
struct FileWriterStats;
class MongoConfig;
class RemoteControlServer;
class RemoteControlServerStatus;
class StatisticDialog;
class TcpStatistic;
class TelnetRemoteControl;
class ClientManagerFsm;

class QElapsedTimer;
class QLabel;
class QSemaphore;
class QThread;

class EvbMainWindow : public BaseMainWindow
{
    Q_OBJECT

public:
    explicit EvbMainWindow(const EvBStartupOptions &so, QWidget * parent = nullptr);
    void init();
    ~EvbMainWindow() override;
    EvbClientStatusTableView * const getStatusTable() const;

public slots:
    void setConfig(const EvBConfig &newConfig);
    void setClientList(const ClientIndexEnabledMap &) {}
    void remoteStartRequest(const RcRunId &rcRunId, const QVariant &startEvNumber);
    void remoteStopRequest(const RcRunId &rcRunId, const QVariant &runStopEvNum);
    void onRemoteStatusUpdated(const RemoteControlServerStatus &rcSerStat);
    void pnpCacheUpdated(const PnpInfoCache &cache);
    void configSelectorChanged(const ConfigIndexName& conf) override;
    void onEvbFsmStateChanged(const FsmState &cur, const FsmState &prev);
    void updateCombinerFsmLabel(const FsmState &cur_state);
    void updateClientFsmLabel(const FsmState &cur_state);
    void startLocalRun();
    void stopLocalRun();
    void doReqStateAction(const FsmState &reqSt);
    void updateClientStatus(const TcpClientStatus &status);
    void combinerError(const QString &errStr);
    void fileWriterStatsChanged(const FileWriterStats &fwStat);
    void setCombinerStats(const CombinerStats&);
    void updateButtonRemoveEnableState() const;

private slots:
    void on_pushButtonAdd_clicked();
    void on_pushButtonRemove_clicked();
    void deviceError(const ClientIndex &index, const QString &errStr); // from clients

//    void addLogHistory(QString log, Qt::GlobalColor color=Qt::black);

    void on_actionChange_storage_dir_triggered();
    void on_actionReset_to_defaults_triggered();
    void on_actionAdvanced_counters_toggled(bool showColumns);
    void on_actionAllow_Remote_Control_toggled(bool checked);
    void data_out_changed(QAction *);
    void dataOutChangedRB();
    void on_actionSet_number_of_outputTcpServers_triggered();
    void on_actionSet_EvB_index_triggered();
    void on_actionSet_ring_buffer_sizes_triggered();
    void on_actionSet_file_split_options_triggered();
    void on_actionShow_statistics_triggered();
    void on_actionEnable_client_time_distribution_toggled(bool en);
    void on_actionSet_monitor_port_number_triggered();
    void on_actionPrint_FSM_input_triggered();
    void on_actionFix_TLV_format_toggled(bool en);
    void on_actionProtect_data_file_while_writing_toggled(bool protect);
    void on_actionEnable_inf_output_toggled(bool en);

//    bool clientsDestroying(int client_id, int timeout, QElapsedTimer *timer);
    void on_actionConfiguration_manager_triggered();

    void applyCache();

signals:
    void configChanged(const EvBConfig &config) const;
    void clientListChanged(const ClientIndexEnabledMap &clients_map) const;
    void switch_to_config(const ConfigIndexName& conf) const;
    void requestState(FsmState state, const QVariant runId = QVariant());
    void destroyClient(const ClientIndex &id) const;
    void destroyAll() const;
    void sendRunError(const RcRunId &runId, QString errStr) const;
    void sendRunWarning(const RcRunId &runId, QString warnStr) const;
    void sendBackPreasure(const RcRunId &runId, bool backPreasure) const;
    void updateClientReadout(ClientIndex index, bool readout) const;
    void analizerListUpdated(const ClientIndexSet &clients,
                             const ClientIndexSet &nonEvBasedclients,
                             const QVector<Analizer *> &list) const;
    void allowRemoteControl(bool) const;
    void updateProgramDescription(const ProgramDescription &) const;
    void closeProgram(const ProgramDescription &) const;
    void resizeOutputBuffer(int outputBufSize) const;
    void setMonitorPortNum(quint16 portNum) const;
    void sendRunDocumentToRc(const RunDocument &rd) const;
    void printFsmInput() const;

signals: void writerConfigChanged(const EvbWriterConfig &) const;
public slots: void setWriterConfig(const EvbWriterConfig &cfg) {}
signals: void evNumberConfigChanged(const EvbEvNumberConfig &) const;
public slots: void setEvbEvNumberConfig(const EvbEvNumberConfig &cfg) {}

private:
    void registerTypes();
    void initCombiner();
    void applyConfig();
    void update_combiner_to_config();
    void updateOutTcpLabelList();
    void checkTrashedEvents(const CombinerStats &cs,
                                       const CombinerStats &prevCs) const;
    void updateCombinerStatusLabel(const CombinerStats &cs) const;
    void updateFileErrLabel(const CombinerStats &cs) const;
    void sendCombinerMetrics(const CombinerStats &cs) const;
    void checkBackPreasure(const CombinerStats &cs);
    void setup_clients();
    void deleteClients(const ClientIndexSet &set);
    bool addClient(const EvBClientConfig &client_config);
    void updateGui() const;
    void updateGuiCombinerOut() const;
    void updateSplitGui() const;
    void updateGuiEnableStates() const;
    void updateButtonAddEnableStates() const;
    void updateAnalizerList();
    void initProgramDescription(const ConfigIndexName &conf);
    void refreshProgramDescription();
    void updateTcpPortLabels();
    void setOutTcpServerNumber(int tcpNum);
    QVector<ProgramDescription> getFreeProgs() const;
    bool hasFreeProgs() const;
    bool isFreeProg(const ProgramDescription &pd) const;
    void combinerOutUpdated();
    QLabel *getTcpLabel(int index) const;
    void updateTcpLabelVisibility() const;
    bool isNonEventBaseClient(const ClientIndex &index) const;
    FsmState getFsmState() const;
    bool isInRun() const;
    void updatePendingProgramDescription();
    void sendRunDocument() const;

    Ui::EvbMainWindow *ui;
    ConfigurationManager * const configuration_manager;
    EvBConfig config;
    EvBConfigDb * const configDb;
    EvBConfigDispatcher * const configDispatcher;
    EventBuilderFsm * const evb_fsm;
    FsmState requestedState = FsmState::Init;
    ClientManagerFsm * const client_manager_fsm;
    QMap<ClientIndex, Client*> clients;
    QHash<ClientIndex, TcpClientStatus> lastTcpClientStatus;
    ClientIndexSet nonEvBasedClients;
    Combiner *combiner;
    QThread *combinerThread;
    bool allOk;
    RemoteControlServer * const remoteControl;
    ProgramDescription progDesc;
    bool run;
    RcRunId loc_runId;
    std::optional<quint32> startEvNum, stopEvNum;
    QLabel * const labelFileErrSize;
    PnpInfoCache progDescrsCache;
    bool hasNewPnpCache = false;
    CombinerStats lastCombinerStats;
    uint enabledClientNum;
    StatisticDialog * const statisticDialog;
    bool backPreasure = false;
    QLabel * const labelEvbFsmState;
    QLabel * const labelCombinerFsmState;
    QLabel * const labelClientManagerFsmState;
    QLabel * const labelCombinerStatus;
    FileSplitDialog * const fileSplitDialog;
    EvbFileNameDialog * const fileNameDialog;
    AdjustBufferDialog * const adjustBufferDialog;
    AdjustBufferModel * const adjustBufferModel;
    QTimer * const applyCacheTimer;
    TelnetRemoteControl * const telnetRc;
};

#endif // EVBMAINWINDOW_H
