#include "EvbSocketStatus.h"
#include <asm-generic/socket.h>
#include <sys/socket.h>

#include <QHostInfo>
#include <QTcpSocket>


#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/select.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <string.h>

void SocketSizeStatus::updateSocketOptions(std::optional<int> s)
{
    if(!s) {
        *this = {};
        return;
    }

    int val;
    socklen_t l = sizeof(val);

    ::getsockopt(*s, SOL_SOCKET, SO_RCVBUF, &val, &l);
    if(l == sizeof (val))
        soReceiveBufferSize = val;
    else
        soReceiveBufferSize.reset();

    l = sizeof(val);
    ::getsockopt(*s, SOL_SOCKET, SO_RCVBUF, &val, &l);
    if(l == sizeof (val))
        soSendBufferSize = val;
    else
        soSendBufferSize.reset();
}

void SocketAddress::updateAddr(int s, bool remote)
{
    struct sockaddr_in addr;
    memset(&addr, 0, sizeof(addr));
    socklen_t len = sizeof(addr);
    auto ret = remote ? getpeername(s, reinterpret_cast<sockaddr *>(&addr), &len)
                      : getsockname(s, reinterpret_cast<sockaddr *>(&addr), &len);
    if(ret)
        return;

    updateAddr(addr);
}

SocketAddress SocketAddress::getLocal(int s)
{
    SocketAddress ret;
    ret.updateAddr(s, false);
    return ret;
}

void SocketAddress::updateAddr(sockaddr_in addr)
{
    ip = ntohl(addr.sin_addr.s_addr);
    if(ip) {
        host = QHostAddress(ip);
        hostName = host.toString();
    } else {
        host.clear();
        hostName.clear();
    }
    port = ntohs(addr.sin_port);
}

QString SocketAddress::toString() const
{
    return (hostName.isEmpty() ? host.toString() : hostName)
            + ":" + QString::number(port);
}

void EvbSocketStatus::updateRingBuffet(RingBuffer<char> *rb)
{
    ringBufSize = rb->getAllSize();
    freePart = ringBufSize ? 100.*rb->getFreeSize()/ringBufSize : 0;
    dataPart = ringBufSize ? 100.*rb->getDataSize()/ringBufSize : 0;
    trashPart = ringBufSize ? 100.*rb->getTrashSize()/ringBufSize : 0;
    isEmpty = rb->getDataSize()==0;
}

void EvbSocketStatus::updateLocalPort(int s)
{
    auto v = SocketAddress::getLocal(s);
    if(v.port)
        localPort = v.port;
    else
        localPort.reset();
    peer.updateAddr(s);
}
