#ifndef EVBCLIENTSTATUSMODEL_H
#define EVBCLIENTSTATUSMODEL_H

#include <QObject>

#include "mvc/ClientIndexTableModel.h"

class ClientIndex;
class ClientIndexEnabledMap;
class EvbClientStatusModelPrivate;
class PnpInfoCache;
struct CombinerStats;
struct TcpClientStatus;

class EvbClientStatusModel : public ClientIndexTableModel
{
    Q_OBJECT
public:
    enum Columns{
        COL_TYPE,
        COL_INDEX,
        COL_DATA_TYPE,
        COL_CONNECTION_STATE, COL_PROG_HOST, COL_STATUS,
        COL_EV_NUM, COL_TRASHED_EVENTS, COL_ALL_EVENTS,
        COL_FLUSHED_SIZE, COL_TIME_DISTR,
        COL_TCP_BUFF,
        COL_EV_BUFF, COL_TOTAL};

    explicit EvbClientStatusModel(QObject *parent = nullptr);
    ~EvbClientStatusModel() override;

    virtual ClientIndex clientIndexOf(int row) const override;
    virtual int rowOf(const ClientIndex &sourceIndex) const override;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;

signals:
    void colHiddenUpdated(int col, bool vis) const;

public slots:
    void setClientList(const ClientIndexEnabledMap &map);
    void setTcpClientStatus(const TcpClientStatus &status);
    void setCombinerStats(const CombinerStats &cs);

private:
    QVariant data_DisplayRole(const ClientIndex &clIndex, int column) const;
    QVariant data_ToolTip(const ClientIndex &clIndex, int column) const;
    QVariant data_ForegroundRole(const ClientIndex &clIndex, int column) const;
    QVariant data_DecorationRole(const ClientIndex &clIndex, int column) const;
    void invalidateRow(const ClientIndex &i, const QVector<int> roles = {});
    bool isNonEventBaseClient(const ClientIndex &index) const;

    Q_DISABLE_COPY_MOVE(EvbClientStatusModel)
    Q_DECLARE_PRIVATE(EvbClientStatusModel)
    EvbClientStatusModelPrivate * const d_ptr;
};

#endif // EVBCLIENTSTATUSMODEL_H
