#include "EvbDebugLabel.h"

#include "TcpClientStatus.h"

EvbDebugLabel::EvbDebugLabel(QWidget *parent, Qt::WindowFlags f)
    : QLabel(parent, f)
{}

void EvbDebugLabel::updateDebugInfoLabel(const QHash<ClientIndex, TcpClientStatus> &lastTcpClientStatus,
                                         const CombinerStats &cs)
{
    if(!repeCombStats) {
        minClient = maxClient = {};
    }
    const CombinerStats &reperCs = repeCombStats ? *repeCombStats
                                                 : lastCombinerStats;
    for(auto it=cs.clientsStat.cbegin(); it!=cs.clientsStat.cend(); ++it) {
        auto index = it.key();
        auto tcpStIt = lastTcpClientStatus.constFind(index);
        if(tcpStIt == lastTcpClientStatus.cend())
            continue;
        auto freePart = tcpStIt->statistic.freePart;
        if(!minClient.second || minClient.second>freePart) {
            minClient.first = index;
            minClient.second = freePart;
        }
        if(!maxClient.second || maxClient.second<freePart) {
            maxClient.first = index;
            maxClient.second = freePart;
        }
    }
    QStringList infoList;
    if(maxClient.second) {
        infoList << QString("minFree: %1% - %2")
        .arg(*minClient.second, 0, 'f', 1)
            .arg(minClient.first.toString());
        infoList << QString("maxFree: %1% - %2")
                        .arg(*maxClient.second, 0, 'f', 1)
                        .arg(maxClient.first.toString());
    }
    auto dt = reperCs.timeStamp.msecsTo(cs.timeStamp);
    std::optional<QPair<double, double>> combRate;
    for(auto it=cs.clientsStat.cbegin();it!=cs.clientsStat.cend();++it) {
        double clRate = it->allBlocks-reperCs.clientsStat.value(it.key()).allBlocks;
        clRate /= dt;
        if(combRate) {
            if(combRate->first>clRate)
                combRate->first=clRate;
            if(combRate->second<clRate)
                combRate->second=clRate;
        } else
            combRate = {clRate, clRate};
    }
    if(combRate) {
        auto s = QString("CombRate: %L1").arg(combRate->first);
        if(!qFuzzyCompare(combRate->first, combRate->second))
            s += QString("%L1").arg(combRate->second);
        infoList << (s+" kHz");
    }
    if(repeCombStats)
        infoList << QString("dt: %L1ms").arg(dt);
    setText(infoList.join('\n'));

    lastCombinerStats = cs;
}

void EvbDebugLabel::mouseDoubleClickEvent(QMouseEvent *event)
{
    QLabel::mouseDoubleClickEvent(event);

    if(repeCombStats)
        repeCombStats.reset();
    else
        repeCombStats = lastCombinerStats;
}
