#include "MscSignalRouter.h"

#include "MscCore.h"
#include "MscDevice.h"
#include "MscDeviceGlobal.h"
#include "MscStatusPublisher.h"

#include "base/DeviceId.h"
#include "config/MscConfigDispatcher.h"
#include "msc-gui/MscHistDialog.h"
#include "msc-gui/MscHistModel.h"
#include "msc-gui/MscMainWindow.h"
#include "msc-gui/MscThresholdModel.h"

#include "base/DeviceIndex.h"
#include "fsm/DeviceMgrFsm.h"
#include "mldiscover/DeviceDispatcher.h"
#include "mldiscover/LldpDiscovery.h"
#include "mstream-lib/MStreamDump.h"
#include "m-stream-output/MStreamStatisticWidget.h"

static constexpr const auto device_filter_list = {
    DEVICE_ID_MSC16VE_E
};

MscSignalRouter::MscSignalRouter()
{
    setDeviceIdFilter(device_filter_list);
    setConfigDispatcher(&MscConfigDispatcher::instance());
}

void MscSignalRouter::setCore(MscCore *p)
{
    core = p;
    setReadoutAppCore(p);
}

void MscSignalRouter::setMainWindow(MscMainWindow *p)
{
    mainWindow = p;
    setReadoutAppMainWindow(p);
}

void MscSignalRouter::connect_MainWindow() const
{
    ReadoutAppSignalRouter::connect_MainWindow();
    Q_ASSERT(core);
    Q_ASSERT(mainWindow);
    Q_ASSERT(histDialog);
    Q_ASSERT(histModel);
    Q_ASSERT(thresholdModel);
    Q_ASSERT(histDialog);
    Q_ASSERT(deviceMgrFsm);

    connect(histDialog, &MscHistDialog::resetConfigHistCh,
            &MscConfigDispatcher::instance(), &MscConfigDispatcher::resetConfigHistCh);

    connect(deviceMgrFsm, &BaseFsm::stateChanged,
            mainWindow, &MscMainWindow::onFsmStateChange);
}

void MscSignalRouter::connect_MscDevice(const MscDeviceGlobal *dg, const MscDevice *dev) const
{
    ReadoutAppSignalRouter::connect_device(dg, dev);

    Q_ASSERT(core);
    Q_ASSERT(deviceDispatcher);
    Q_ASSERT(statusPublisher);
    Q_ASSERT(deviceMgrFsm);

    const auto *device = dg->getDevice<MscDevice>();
    deviceDispatcher->subscribeHostAddressUpdates(dg->getDeviceIndex(), device);

    connect(device, &MscDevice::statusUpdated,
            dg, &MscDeviceGlobal::statusUpdated);

    connect(device, &MscDevice::statusUpdated,
            core, &MscCore::statusUpdated);

    connect(device, &MscDevice::buildParametersUpdated,
            &MscConfigDispatcher::instance(),
            &MscConfigDispatcher::setBuildParameters);

    connect(device, &MscDevice::histUpdated,
            statusPublisher, &MscStatusPublisher::histUpdated);

    if (mainWindow) {
        connect(device, &MscDevice::statusUpdated,
                mainWindow, &MscMainWindow::statusUpdated);
        connect(device, &MscDevice::statusUpdated,
                thresholdModel, &MscThresholdModel::statusUpdated);
        connect(device, &MscDevice::deviceConnectedFw,
                mainWindow, &MscMainWindow::deviceConnected);
        connect(device, &MscDevice::deviceDisconnected,
                mainWindow, &MscMainWindow::deviceDisconnected);
        connect(device, &MscDevice::histUpdated,
                mainWindow, &MscMainWindow::histUpdated);
    }
    connect(device, &MscDevice::cycleStatusUpdated,
            histModel, &MscHistModel::cycleStatusUpdated);
    if (mstreamStatisticWidget) {
        connect(dg, &MstreamMregDevGlobal::stat_updated,
                mstreamStatisticWidget, &MStreamStatisticWidget::stat_updated);
        connect(dg, &MstreamMregDevGlobal::stat_from_receiver_updated,
                mstreamStatisticWidget, &MStreamStatisticWidget::stat_from_receiver_updated);
    }

    auto configDispatcher = &MscConfigDispatcher::instance();
    configDispatcher->forwardDeviceConfig(dev);
}

void MscSignalRouter::connect_FSM() const
{
    ReadoutAppSignalRouter::connect_FSM();

    Q_ASSERT(deviceMgrFsm);
    Q_ASSERT(statusPublisher);

    if(statusPublisher) {
        connect(deviceMgrFsm, &DeviceMgrFsm::stateChanged,
                statusPublisher, &MscStatusPublisher::onDeviceMgrFsmStateChanged);
    }
}
