/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TRIGHISTVIEW_H
#define TRIGHISTVIEW_H

#include <QWidget>

namespace Ui {
class TrigHistView;
}

struct FirmwareBuildParams;
struct RcConfig;
class RcConfigDispatcher;
struct SpillPlotData;
struct TriggerHistogramData;

class TrigHistView : public QWidget
{
    Q_OBJECT

public:
    explicit TrigHistView(QWidget *parent = nullptr);
    ~TrigHistView();
    void setConfigDispatcher(RcConfigDispatcher *p);

public slots:
    void setConfig(const RcConfig &newConfig);
    void updatePlotTypeList(quint8 trigSrcCnt);
    void setFirmwareBuildParams(const FirmwareBuildParams &fbp);

signals:
    void configChanged(const RcConfig &config);
    void spillPlotUpdated(const SpillPlotData &);
    void triggerHistogramUpdated(const TriggerHistogramData &);
    void plotChUpdated(int ch) const;

private slots:
    void on_comboBoxSpillStep_activated(int index);
    void on_comboBoxHistSelect_activated(int index);
    void on_comboBoxPlotType_currentIndexChanged(int);

private:
    void restoreGuiFromConfig();
    void initHistWidget();

    Ui::TrigHistView *ui;
    RcConfigDispatcher * configDispatcher = nullptr;
    RcConfig * const config;
};

#endif // TRIGHISTVIEW_H
