//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "VmeTrigsWidget.h"
#include "ui_VmeTrigsWidget.h"

#include "RcConfig.h"
#include "VmeTrigsController.h"
#include "VmeTrigsModel.h"
#include "device-discover/DiscoverDialog.h"
#include "mldiscover/DeviceDispatcher.h"
#include "mldiscover/LldpDiscovery.h"

#include <QAbstractTableModel>
#include <QCheckBox>
#include <QDebug>
#include <QElapsedTimer>
#include <QHostAddress>
#include <QInputDialog>
#include <QMessageBox>
#include <QTimer>

Q_DECLARE_METATYPE(QHostAddress)

VmeTrigsWidget::VmeTrigsWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::VmeTrigsWidget),
    model(new VmeTrigsModel(this)),
    discoverDialog(new DiscoverDialog(this))
{
    ui->setupUi(this);
    qRegisterMetaType<DeviceIndex>();

    discoverDialog->setObjectName("VmeTrigsDiscoverDialog");
    auto table = ui->tableWidgetVmeTrig;
    table->setModel(model);
    table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    table->hideColumn(VmeTrigsModel::COL_VME_ID);
    table->hideColumn(VmeTrigsModel::COL_VME_CHECK_EV_NUM);
    table->setSelectionMode(QAbstractItemView::NoSelection);

    qRegisterMetaType<QHostAddress>();
}

VmeTrigsWidget::~VmeTrigsWidget()
{
    delete ui;
}

void VmeTrigsWidget::setDeviceDispatcher(DeviceDispatcher *p)
{
    if (deviceDispatcher) {
        discoverDialog->disconnect(deviceDispatcher);
        model->disconnect(deviceDispatcher);
    }
    deviceDispatcher = p;
    if (!p)
        return;

    deviceDispatcher->subscribeDevices(discoverDialog);
}

void VmeTrigsWidget::setController(VmeTrigsController *p)
{
    if (controller)
        model->disconnect(controller);
    controller = p;
    if (!p)
        return;

    connect(model, &VmeTrigsModel::trigConfigChanged,
            controller, &VmeTrigsController::setTrigConfig);
    connect(controller, &VmeTrigsController::trigConfigChanged,
            model, &VmeTrigsModel::setTrigConfig);

    connect(controller, &VmeTrigsController::deviceAdded,
            model, &VmeTrigsModel::addDevice);
    connect(controller, &VmeTrigsController::deviceRemoved,
            model, &VmeTrigsModel::removeDevice);
    connect(controller, &VmeTrigsController::deviceStatusUpdated,
            model, &VmeTrigsModel::setTrigStatus);
    connect(controller, &VmeTrigsController::stateChanged,
            this, &VmeTrigsWidget::setRunState);

    const auto id_filter = controller->getDeiveIdFilter();
    auto &lldpDiscovery = LldpDiscovery::instance();
    lldpDiscovery.subscribe(discoverDialog, id_filter);
    lldpDiscovery.subscribe(model, id_filter);
}

void VmeTrigsWidget::setRunState(const FsmState &state)
{
    fsmState = state;
    updateEnableState();
}

bool VmeTrigsWidget::isCfgChangeAllowed() const
{
    return fsmState == FsmState::Error || fsmState == FsmState::Ready;
}

void VmeTrigsWidget::updateEnableState()
{
    const bool enCfg= isCfgChangeAllowed();
    discoverDialog->setEnabled(enCfg);
    model->setEnabled(enCfg);
}

void VmeTrigsWidget::on_pushButtonAddVmeTrig_clicked()
{
    discoverDialog->show();
}

