//
//    Copyright 2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef WREPSTATE_H
#define WREPSTATE_H

#include <QtGlobal>
class QDebug;
class QString;

typedef union {
    quint16 all;
    struct {
        unsigned int sfp: 1;
        unsigned int link: 1;
        unsigned int sync: 1;
        unsigned int reserved: 13;
    } bit;
} reg_wr_status_t;

union reg_wr_tai_code_t {
    quint64 all;
    struct {
        unsigned int bit_0: 1;
        unsigned int valid: 1;
        unsigned int bit_2: 1;
        unsigned int bit_3: 1;
        unsigned int tai_nsec: 28;
        unsigned int tai_sec: 32;
    } bit;
};

struct WrStatusBits
{
    explicit WrStatusBits(quint32 init = 0) {code.all = init;}
    QString toString() const;
    reg_wr_status_t code;
};

QDebug &operator<<(QDebug stream, const WrStatusBits &tai);

struct WrTaiCode
{
    explicit WrTaiCode(quint64 init = 0) {code.all = init;}
    QString toString() const;
    reg_wr_tai_code_t code;
};

QDebug &operator<<(QDebug stream, const WrTaiCode &tai);

enum {
    // WR Endpoint Status
    REG_WR_STATUS              = 0x60,
    REG_WR_RESERVED            = 0x61,
    REG32_WR_UPTIME            = 0x62,
    REG64_WR_TAI_CODE          = 0x64,
    REG32_WR_LINK_LOSS_COUNT   = 0x68,
    REG32_WR_SYNC_LOSS_COUNT   = 0x6A,
    REG32_WR_CODE_ERROR_COUNT  = 0x6C,
    REG32_WR_OTHER_ERROR_COUNT = 0x6E,
    REG64_WR_TAI_TS_1          = 0x70,
    REG64_WR_TAI_TS_2          = 0x74,
    REG64_WR_TAI_TS_3          = 0x78,
    REG64_WR_TAI_TS_4          = 0x7C,
};

#endif // WREPSTATE_H
