#include "CalibrationDialog.h"
#include "ui_CalibrationDialog.h"
#include "CalibrationModel.h"
#include "fsm/FsmState.h"
#include "remote-control-server/RemoteControlServerStatus.h"

#include <QDebug>

CalibrationDialog::CalibrationDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CalibrationDialog),
    model(new CalibrationModel(this))
{
    ui->setupUi(this);
    ui->treeView->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
    ui->treeView->setModel(model);
    ui->treeView->header()->setStretchLastSection(false);
    setSizeGripEnabled(true);
}

CalibrationDialog::~CalibrationDialog()
{
    delete ui;
}

void CalibrationDialog::reset()
{
    model->reset();
}

void CalibrationDialog::onFsmStateChange(const FsmState &state)
{
    ui->pushButtonCalibrate->setEnabled(state == FsmState::Ready && !underRemoteCtrl);
    ui->pushButtonClear->setEnabled(state == FsmState::Ready);
}

void CalibrationDialog::onRemoteStatusUpdated(const RemoteControlServerStatus &rcState)
{
    underRemoteCtrl = rcState.portInUse;
}

void CalibrationDialog::on_pushButtonCalibrate_clicked()
{
    emit startCalibration();
}

void CalibrationDialog::on_pushButtonClear_clicked()
{
    model->clear();
}

void CalibrationDialog::on_pushButtonClose_clicked()
{
    close();
}
