/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TqdcAd5328ConfigBuilder.h"
#include "TqdcDacConverter.h"

namespace {

static const int CHIP_COUNT = 2;
static const int CHIP_CHANNELS = 8;

static constexpr const int TQDC_CH_TO_DAC_CH_MAP[16] = {
    3,2,1,0, 4,5,6,7, 0,1,2,3, 7,6,5,4
};
static constexpr const int TQDC_CH_TO_DAC_INDEX[16] = {
    0, 0, 0, 0,  0, 0, 0, 0,  1, 1, 1, 1,  1, 1, 1, 1
};
} // namespace

Ad5328ConfigBuilder::ChannelIndex TqdcAd5328ConfigBuilder::mapChannel(int channel) const
{
    ChannelIndex channelIndex;
    if (channel < 0 || channel >= CHIP_COUNT * CHIP_CHANNELS)
        return {};
    channelIndex.chip = TQDC_CH_TO_DAC_INDEX[channel];
    channelIndex.chipChannel = TQDC_CH_TO_DAC_CH_MAP[channel];
    return channelIndex;
}

Ad5328Config TqdcAd5328ConfigBuilder::build(const QMap<int, int> &values) const
{
    Ad5328Config config;

    for (auto it = values.begin(); it != values.end(); ++it) {
        int ch = it.key();
        auto dacValue = it.value();
        auto index = mapChannel(ch);
        auto &chip = config.chips[index.chip];
        chip.doubleGain = TqdcDacConverter::instance().getGain();
        chip.dacValue[index.chipChannel] = dacValue;
    }
    return config;
}
