#ifndef TDCVIEWERCONFIGDISPATCHER_H
#define TDCVIEWERCONFIGDISPATCHER_H

#include "AfiViewerConfig.h"
#include "base/DeviceIndexEnabled.h"
#include "lib-common/BaseConfigDispatcher.h"

class AfiViewerConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
public:
    explicit AfiViewerConfigDispatcher(QObject *parent = nullptr);

    virtual void sendCache() override{
        BaseConfigDispatcher::sendCache();
        emit configChanged(*cache);
        emit histConfigChanged(cache->hist_manager_config);
    }

    template<class T>
    void subscribe(T *obj) {
            connect(this, &AfiViewerConfigDispatcher::configChanged,
                    obj, &T::setConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::configChanged,
                    this, &AfiViewerConfigDispatcher::setConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "TdcViewerConfig");
    }

    template<class T>
    void subscribeFileReaderCfg(T *obj) {
            connect(this, &AfiViewerConfigDispatcher::fileReaderConfigChanged,
                    obj, &T::setFileReaderConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::fileReaderConfigChanged,
                    this, &AfiViewerConfigDispatcher::setFileReaderConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "FileReaderConfig");
    }

    template<class T>
    void subscribeHistCfg(T *obj) {
            connect(this, &AfiViewerConfigDispatcher::histConfigChanged,
                    obj, &T::setHistConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::histConfigChanged,
                    this, &AfiViewerConfigDispatcher::setHistConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "HistMngrCfg");
    }

    template<class T>
    void subscribeWaveformViewerCfg(T *obj) {
            connect(this, &AfiViewerConfigDispatcher::waveformViewerConfigChanged,
                    obj, &T::setWaveformViewerConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::waveformViewerConfigChanged,
                    this, &AfiViewerConfigDispatcher::setWaveformViewerConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "HistMngrCfg");
    }

    template<class T>
    void subscribeZmqCfg(T *obj) {
            connect(this, &AfiViewerConfigDispatcher::zmqConfigChanged,
                    obj, &T::setZmqConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::zmqConfigChanged,
                    this, &AfiViewerConfigDispatcher::setZmqConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "ZmqConfig");
    }

signals:
    void configChanged(const AfiViewerConfig &) const;
    void fileReaderConfigChanged(const FileReaderConfig &) const;
    void histConfigChanged(const HistMngrCfg &) const;
    void waveformViewerConfigChanged(const WaveformViewerConfig &) const;
    void zmqConfigChanged(const ZmqConfig &) const;

public slots:
    void setConfig(const AfiViewerConfig &newConfig);
    void setFileReaderConfig(const FileReaderConfig &newConfig);
    void setHistConfig(const HistMngrCfg &newConfig);
    void setWaveformViewerConfig(const WaveformViewerConfig &newConfig);
    void setZmqConfig(const ZmqConfig &newConfig);

protected:
    DeviceIndexEnabledMap getDeviceIndexEnabledMap() const override { return {}; }

private:
    std::unique_ptr<AfiViewerConfig> cache;
};

#endif // TDCVIEWERCONFIGDISPATCHER_H
