#include "HrbCalib.h"

#include <QSettings>
#include <QDebug>

namespace  {
const QString HRB_GROUP_PREFIX = "HrbCal_";
}

HrbCalib::HrbCalib()
{
//    for(uint i=0; i<10; ++i)
//        cal[HRB_GROUP_PREFIX+"test_i"][i] = i;
//    for(uint i=0; i<72; ++i)
//        cal[HRB_GROUP_PREFIX+"test_1"][i] = 1;
//    for(uint i=0; i<72; ++i)
//        cal[HRB_GROUP_PREFIX+"test_0"][i] = 0;
}

void HrbCalib::clear()
{
    cal.clear();
    cal["empty"];
}

void HrbCalib::load()
{
    QSettings s;
    load(&s);
}

bool HrbCalib::load(const QString &fn)
{
    QSettings s(fn, QSettings::IniFormat);
    return load(&s);
}

bool HrbCalib::load(QSettings *s)
{
    bool allOk=true;
    for(auto group : s->childGroups()) {
        if(!group.startsWith(HRB_GROUP_PREFIX)) continue;
        auto presetName = group.right(group.size()-HRB_GROUP_PREFIX.size());
        if(presetName.isEmpty()) continue;
        qInfo().noquote() << QString("Uploading HRB preset '%1' from file:%2")
                             .arg(presetName).arg(s->fileName());

        s->beginGroup(group);
        bool hasReplacement=false;
        for(auto key : s->childKeys()) {
            bool ok;
            int ch = key.toInt(&ok);
            if(!ok) continue;
            double chVal = s->value(key).toDouble(&ok);
            if(!ok) continue;
            if(!hasReplacement && cal[presetName].contains(ch)) {
                qWarning().noquote() << QString("Replacement of HRB calibration '%1' with data from file:%2")
                                        .arg(presetName).arg(s->fileName());
                allOk = false;
                hasReplacement = true;
            }
            cal[presetName][ch] = chVal;
        }
        s->endGroup();
    }
    return allOk;
}

void HrbCalib::save() const
{
    QSettings s;
    for(auto group : s.childGroups()) {
        if(!group.startsWith(HRB_GROUP_PREFIX)) continue;
        s.remove(group);
    }

    for(auto presetName : cal.keys()) {
        if(presetName.isEmpty()) continue;
        auto group = HRB_GROUP_PREFIX+presetName;
        s.beginGroup(group);
        for(auto ch : cal[group].keys()) {
            s.setValue(QString("%1").arg(ch, 2, 10, QChar('0')), cal[group][ch]);
        }
        s.endGroup();
    }
}
