/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef PROGRAMINDEX_H
#define PROGRAMINDEX_H

#include <QHash>
#include <QMetaType>
#include <QString>

class ProgramIndex {
public:
    friend class ProgramIndexConverter;

    ProgramIndex() { unify(); }
    ProgramIndex(const QString &domain, const QString &index)
        : domain(domain), index(index) { unify(); }

    friend bool operator==(const ProgramIndex &l, const ProgramIndex &r) = default;
    friend bool operator<(const ProgramIndex &l, const ProgramIndex &r)
    {
        if (l.domain == r.domain)
            return l.index < r.index;
        return l.domain < r.domain;
    }
    friend uint qHash(const ProgramIndex &r, uint seed = 0) {
        return qHash(r.domain) ^ qHash(r.index) ^ seed;
    }
    friend QDataStream &operator<<(QDataStream &out, const ProgramIndex &value);
    friend QDataStream &operator>>(QDataStream &in, ProgramIndex &value);
    friend QDebug operator<<(QDebug dbg, const ProgramIndex &r);

    bool isNull() const { return domain.isNull() && index.isNull();}
    bool isEmpty() const { return domain.isEmpty() && index.isEmpty();}
    QString getDomain() const { return domain.isEmpty() ? _local : domain; }
    QString getIndex() const { return index.isEmpty() ? _default : index; }

    bool isDefault() const { return isDomainDefault() && isIndexDefault(); }
    bool isDomainDefault() const { return getDomain() == _local; }
    bool isIndexDefault() const { return getIndex() == _default; }
    // static ProgramIndex defaultValue() { return ProgramIndex(initValue); }
    QString toString() const;
    static ProgramIndex fromString(const QString &r);

private:
    void unify() {
        if(getIndex() == _default)
            index = _default;
        if(getDomain() == _local)
            domain = _local;
    }

    static constexpr const char *_default = "default";
    static constexpr const char *_local = "local";
    QString domain;
    QString index;
};
Q_DECLARE_METATYPE(ProgramIndex)

#endif // PROGRAMINDEX_H
