/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RunIdConverter.h"

namespace {
constexpr const char *k_runIndex = "runIndex";
constexpr const char *k_runNumber = "runNumber";
constexpr const char *k_uuid = "uuid";
}

template<>
bool AbstractConfigConverter<RcRunId>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RcRunId>::toJsonObject(const RcRunId &in)
{
    QVariantMap m;
    m[k_runIndex] = QVariant::fromValue(in.runIndex);
    m[k_runNumber] = QVariant::fromValue(in.runNumber);
    m[k_uuid] = QVariant::fromValue(in.uuid.toString());
    return fromVariantMap(m);
}

template<>
RcRunId AbstractConfigConverter<RcRunId>::fromJsonObject(const QJsonObject &m)
{
    return RcRunId(m[k_runIndex].toString(),
        m[k_runNumber].toInt(),
        QUuid(m[k_uuid].toString())
                   );
}

template<>
QJsonValue AbstractConfigConverter<RcRunId>::toJsonValue(const RcRunId &in)
{
    return toJsonObject(in);
}

template<>
RcRunId AbstractConfigConverter<RcRunId>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RcRunId>::registerSubTypes()
{
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    if (QMetaType::hasRegisteredComparators<RcRunId>())
        return;
    qRegisterMetaTypeStreamOperators<RcRunId>();
    QMetaType::registerComparators<RcRunId>();
#endif
//    QMetaType::registerConverter<RcRunId, QString>(&toString);
//    QMetaType::registerConverter<QString, RcRunId>(&fromString);
}

//QString RunIdConverter::toString(const RcRunId &r)
//{
//    return r.runIndex + ":" + QString::number(r.runNumber);
//}

//RcRunId RunIdConverter::fromString(const QString &r)
//{
//    const auto list = r.split(":");
//    if (list.size() < 2)
//        return RcRunId();
//    QString runIndex = list[0];
//    bool ok;
//    int runNumber = list[1].toInt(&ok);
//    if (!ok)
//        return RcRunId();

//    return RcRunId(runIndex, runNumber);
//}

