/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "util.h"

#include <algorithm>

#include <QAssociativeIterable>
#include <QObject>
#include <QString>
#include <QStringList>
#include <QDebug>
#include <QJsonDocument>
#include <QVariant>

QString qObjectAncestryStr(const QObject *obj)
{
    QStringList sl;
    for (const QObject *p = obj; p; p = p->parent()) {
        const QString &classStr = p->metaObject()->className();
        const QString &objStr = p->objectName();
        QString s = classStr;
        if (! objStr.isEmpty() && (objStr != classStr))
            s += QString("[%2]").arg(objStr);
        sl += s;
    }
    std::reverse(std::begin(sl), std::end(sl));
    return sl.join(".");
}

QDebug operator<<(QDebug dbg, const QJsonDocument &doc)
{
    const char *endl = "\n";
    dbg.noquote().nospace() << endl << doc.toJson();
    return dbg;
}

void recursive_compare(const QVariant &l, const QVariant &r)
{
//    if (l.canConvert<QVariantList>()) {
//        QSequentialIterable iterable = l.value<QSequentialIterable>();
//        for (const QVariant &v : iterable) {
//            //
//        }
//    }
    if (l.type() != r.type()){
        qDebug() << "Type mismatch:" << l << r;
        return;
    }
    if (l.canConvert<QVariantHash>()) {
        QAssociativeIterable iterable = l.value<QAssociativeIterable>();
            for (auto it = iterable.begin(); it != iterable.end(); ++it) {
                auto key = it.key();
                auto value_l = it.value();
                auto value_r = r.value<QAssociativeIterable>().value(key);
                recursive_compare(value_l, value_r);
            }
            return;
    }
    if (l != r) {
        const char *endl = "\n";
        qDebug().noquote().nospace() << "Value mismatch:"
                                     << endl << "  left:  " << l
                                     << endl << "  right: " << r;
    }
}

double convertOneWireTemp(quint16 rawTemp)
{
    rawTemp &= 0xFFF;
    if (rawTemp & 0x800) rawTemp = -(rawTemp&0x7FF);
    return 0.0625 * rawTemp;
}
