#include "CalResult.h"
#include <cmath>

MultichannelBaselineStats::MultichannelBaselineStats(const QMap<int, QVector<int>> &allChannelData)
{
    if (allChannelData.empty())
        return;
    for (auto channel_it = allChannelData.begin(); channel_it != allChannelData.end(); ++channel_it) {
        auto ch = channel_it.key();
        double mean = 0;
        // const auto &samples = channel_it.value();
        auto samples = channel_it.value();
        if (samples.empty())
            continue;
        std::sort(samples.begin(), samples.end());
        auto size = samples.size();
        BaselineStats stats;
        stats.sampleCount = size;
        stats.min = samples.first();
        stats.max = samples.last();
        stats.median = samples[samples.size() / 2];
        if (size >= 2) {
//            double medianTrim = 0.10;
//            int trimCount = size * medianTrim;
//            if (size*medianTrim > 2) {
//                samples.remove(0, trimCount);
//                samples.remove(samples.size()-1-trimCount, trimCount);
//            }
            for (auto sample : qAsConst(samples))
                mean += sample;
            mean /= samples.size();
            stats.mean = mean;
            double variance = 0;
            for (auto sample : qAsConst(samples))
                variance += pow((sample - mean), 2);
            stats.stddev = sqrt(variance / samples.size());
        }
        insert(ch, stats);
    }
}

QString BaselineStats::toString() const
{
    return QString("mean %1, median %2, min %3, max %4, stddev %5, peak_factor %6")
        .arg(mean).arg(median).arg(min).arg(max).arg(stddev).arg(peakFactor());
}

QString MultichannelBaselineStats::toString() const
{
    auto width = 12;
    auto columns = {"channel", "mean", "median", "min", "max", "stddev", "peak_factor"};
    QMap<int, QVector<double>> values;
    for (auto it = cbegin(); it != cend(); ++it) {
        const auto &v = it.value();
        values[it.key()] = {v.mean, v.median, v.min, v.max, v.stddev, v.peakFactor()};
    }
    QStringList sl;
    QString header;
    for (auto it = columns.begin(); it != columns.end(); ++it) {
        header += QString("%1").arg(*it, width);
    }
    sl << header;
    for (auto chIt = values.cbegin(); chIt != values.cend(); ++chIt) {
        QString s = QString("%1").arg(chIt.key(), width);
        const auto &v = chIt.value();
        for (auto it = v.cbegin(); it != v.cend(); ++it) {
            s += QString("%1").arg(*it, width, 'f', 1);
        }
        sl << s;
    }
    return sl.join("\n");
}

QMap<int, double> MultichannelBaselineStats::meanMap() const
{
    QMap<int, double> result;
    for (auto it = cbegin(); it != cend(); ++it) {
        const auto &v = it.value();
        result[it.key()] = v.mean;
    }
    return result;
}

QMap<int, double> MultichannelBaselineStats::rmsMap() const
{
    QMap<int, double> result;
    for (auto it = cbegin(); it != cend(); ++it) {
        const auto &v = it.value();
        result[it.key()] = v.stddev;
    }
    return result;
}
