/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "LogViewWidget.h"

#include <QTime>

#include "base/SyslogMessageSender.h"

LogViewWidget::LogViewWidget(QWidget *parent) : QPlainTextEdit(parent)
{
    // log message may be sent before inherited class constructor has finished,
    // should use QueuedConnection
    connect(&SyslogMessageSender::instance(), &SyslogMessageSender::logMessage,
            this, &LogViewWidget::addLogMessage,
            Qt::QueuedConnection);
}

LogViewWidget::~LogViewWidget()
{
    SyslogMessageSender::instance().disconnect(this);
}


void LogViewWidget::addLog(QtMsgType severity, const QString &log, const QString &thrName)
{
    addLogMessage(severity, QString(), log, thrName);
}

void LogViewWidget::addLogMessage(QtMsgType severity, const QString &category, const QString &log, const QString &threadName)
{
    Qt::GlobalColor color = Qt::black;
    switch (severity) {
    case QtCriticalMsg:
        color = Qt::magenta;
        break;
    case QtWarningMsg:
        color = Qt::red;
        break;
    case QtInfoMsg:
        color = Qt::blue;
        break;
    case QtDebugMsg:
#ifndef QT_DEBUG
        return;
#endif
        color = Qt::darkGray;
        break;
    case QtFatalMsg:
        break;
    }

    QTextCharFormat tf = currentCharFormat();
    tf.setForeground(QBrush(color));
    setCurrentCharFormat(tf);
    QString str = log;
    if (!category.isEmpty() && category != "default")
        str.prepend(QString("%1: ").arg(category));
    if(!threadName.isEmpty())
        str.prepend(QString("[%1] ").arg(threadName));
    str.prepend(QString("[%1] ").arg(QTime::currentTime().toString()));

    appendPlainText(str);
//    appendPlainText(QString("[%1] [%2] %3")
//                                          .arg(QTime::currentTime().toString(), thrName, log));
}
