#include "FeLinkStatus.h"

#include <QDebug>

QDebug operator<<(QDebug dbg, const FeLinkChannelInfo &v)
{
    dbg
            << "ch:" << v.ch << "; "
            << "slotNum:" << v.slotNum << "; "
            << "isMaster:" << v.isMaster << "; "
               ;

    return dbg;
}

QDebug operator<<(QDebug dbg, const FeLinkStatus &c)
{
    dbg
            << "txInfo:" << c.txInfo << "; "
            << "rxInfo:" << c.rxInfo << "; "
            << "communication_evn:" << c.communication_evn << "; "
            << "baseStatus:" << c.baseStatus << "; "
            << "sfpStatus:" << c.sfpStatus << "; "
            << "phase_adjust_enabled:" << c.phase_adjust_enabled << "; "
            << "RX_Bitslide:" << c.RX_Bitslide << "; "

            << "feLinkVer:" << c.feLinkVer << "; "
            << "remoteVer:" << c.remoteVer << "; "

            << "remoteIndex:" << c. remoteIndex << "; "
            << "REG_FELINK_SYNC_RESEND_TIME:" << c.REG_FELINK_SYNC_RESEND_TIME << "; "
            << "REG_FELINK_TIME_FRAME_TIMEOUT:" << c.REG_FELINK_TIME_FRAME_TIMEOUT << "; "

//            << "gtx_status:" << c.gtx_status << "; "

            << "timeCode:" << c.timeCode << "; "

            << "Current_link_state_time:" << c.Current_link_state_time << "; "

            << "RX_code_error_counter:" << c.counter.RX_code_error << "; "
            << "RX_CRC_error_counter:" << c.counter.RX_CRC_error << "; "
            << "Time_frames_sent_counter:" << c.counter.time_fr_sent << "; "
            << "Time_frames_received_counter:" << c.counter.time_fr_received << "; "
            << "Trigger_frames_sent_counter:" << c.counter.trigger_fr_sent << "; "
            << "Trigger_frames_received_counter:" << c.counter.trigger_fr_received << "; "
            << "Trigger_frames_sent_counter:" << c.counter.trigger_fr_sent << "; "
            << "Trigger_frames_received_counter:" << c.counter.trigger_fr_received << "; "
            << "Unknown_frames_received_counter:" << c.counter.unknown_received << "; "
            << "phy_reset:" << c.counter.phy_reset
               ;

    return dbg;
}

QDebug operator<<(QDebug dbg, const FeLinkBaseStatus &v)
{
    dbg
        << "PHY_level_sync_accure:" << v.PHY_level_sync_accure << "; "
        << "rx_sync:" << v.rx_sync << "; "
        << "remote_rx_sync:" << v.remote_rx_sync << "; "
        << "softpll_locked:" << v.softpll_locked << "; "
        << "link_ok:" << v.link_ok << "; "
        << "remote_link_ok:" << v.remote_link_ok << "; "
        ;

    return dbg;
}

QDebug operator<<(QDebug dbg, const FeLinkSfpStatus &v)
{

    dbg
            << "sfp_inserted:" << v.inserted << "; "
            << "sfp_sig_detected:" << v.sig_detected << "; "
            << "sfp_tx_fault:" << v.tx_fault << "; "
            << "sfp_tx_disable:" << v.tx_disable << "; "
               ;

    return dbg;
}

QDebug operator<<(QDebug dbg, const FeLinkProtocolVersion &c)
{
    dbg
            << "maj:" << c.maj << "; "
            << "min:" << c.min
               ;

    return dbg;
}

QString FeLinkProtocolVersion::getInfoStr() const
{
    return QString("%1.%2").arg(maj).arg(min);
}

QString FeLinkStatus::getInfoStr(bool shortType) const
{
    QStringList list;
    if(!baseStatus.PHY_level_sync_accure)
        list << (shortType ? "PhySync" : "no PHY_sync");
    if(!baseStatus.rx_sync)
        list << (shortType ? "RxSyn" : "rx_sync fail");
    if(!baseStatus.remote_rx_sync)
        list << (shortType ? "RmRxSyn" : "remote_rx_sync fail");
    if(!txInfo.isMaster && phase_adjust_enabled && !baseStatus.softpll_locked)
        list << (shortType ? "SoftPll" : "softpll_locked fail");

    static bool warn_incorrect_link_ok = true;
    if(warn_incorrect_link_ok && baseStatus.link_ok != list.isEmpty()) {
        warn_incorrect_link_ok = false;
        qWarning() << __PRETTY_FUNCTION__ << "warn_incorrect_link_ok status:" << *this;
    }

    if(baseStatus.link_ok) {
        list << "link_ok";
    } else {
        if(list.isEmpty())
            list << "link_fail";
    }

    if(shortType)
        return  list.first();
    else {
        list << (baseStatus.remote_link_ok ? "remote_link - ok" : "remote_link fail");
        return  list.join(",");
    }
}

QString FeLinkStatus::getSfpStatusStr(bool full) const
{
    QStringList l;
    if(full) {
        l << QString("%1%2%3%4")
             .arg(sfpStatus.tx_disable ? "1" : "_")
             .arg(sfpStatus.tx_fault ? "1" : "_")
             .arg(sfpStatus.sig_detected ? "1" : "_")
             .arg(sfpStatus.inserted ? "1" : "_");
        l << "___X - SFP inserted (inverted SFP_MOD_DETECT)";
        l << "__X_ - SFP Signal Detect (inverted SFP_LOS)";
        l << "_X__ - SFP_TX_FAULT";
        l << "X___ - SFP_TX_DISABLE";
    } else {
        if(!sfpStatus.inserted)
            return "no SFP";
        if(sfpStatus.sig_detected)
            l << "no signal";
        if(sfpStatus.sig_detected)
            l << "TX FAULT";
        if(sfpStatus.tx_disable)
            l << "TX DISABLE";
        if(l.isEmpty())
            return "SFP ok";
    }

    return l.join("\n");
}

QString FeLinkStatus::getCommunicationEnvStr(bool remote) const
{
    QString name;
    if(!remote) {
        switch (communication_evn) {
        case FE_LINK_COM_ENV_QSFP: name = "QSFP"; break;
        case FE_LINK_COM_ENV_SFP: name = "SFP"; break;
        case FE_LINK_COM_ENV_VXS: name = "VXS"; break;
        case FE_LINK_COM_ENV_OTHER: return "Other";
        default: Q_UNREACHABLE();
        }
    }

    QString suf;
    const auto &chInfo = remote ? rxInfo : txInfo;
    if(communication_evn == FE_LINK_COM_ENV_QSFP) {
        suf = QString("%1.%2")
                  .arg(chInfo.getSfpConnector().first)
                  .arg(chInfo.getSfpConnector().second);
    } else {
        suf = QString::number(chInfo.slotNum);
    }

    return QString("%1 #%2").arg(name).arg(suf).trimmed();
}

bool FeLinkBaseStatus::allOk() const
{
    return PHY_level_sync_accure
            && rx_sync
            && remote_rx_sync
            && link_ok;
}
