/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef FLASHDEVIMPL_H
#define FLASHDEVIMPL_H

#include "flash-prog/FlashDev.h"

class AbstractEasyDeviceModule;
class DeviceIndex;

class FlashDevImpl : public FlashDev
{
public:
    explicit FlashDevImpl(AbstractEasyDeviceModule *devModule, QObject *parent = nullptr) :
        FlashDev(parent), devModule(devModule)
    {}

protected:
    QString getPrefix() const override;

    bool devReadReg(int regAddr, uint16_t *result) override;
    bool devWriteReg(int regAddr, uint16_t data) override;
    bool devWriteReg32(int regAddr, uint32_t data) override;
    bool devBlkWriteReg(uint count, const uint *addr, uint16_t *data) override;
    bool devBlkReadReg(uint count, const uint *addr, uint16_t *data) override;

private:
    DeviceIndex getDeviceIndex() const;
    AbstractEasyDeviceModule * const devModule;
};

#endif // FLASHDEVIMPL_H
