/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ModularDeviceConfigConverter.h"
// #include "base/CoreTypeConverter.h"
#include "ModularDeviceConfig.h"
// modules
#include "device-modules/AdcErc/AdcErcConfigConverter.h"
#include "device-modules/DeserializersControl/AdcSerDesDelayConverter.h"
#include "device-modules/ClockControl/ClockControlConfigConverter.h"
#include "device-modules/FlashProg/FlashProgModuleConfigConverter.h"
#include "device-modules/NetworkPort/NetworkPortConfigConverter.h"
#include "device-modules/RunLogic/RunLogicConfigConverter.h"
#include "device-modules/SpillControl/SpillConfigConverter.h"
#include "device-modules/StatReadout/StatReadoutConfigConverter.h"
#include "device-modules/TrigCtrlStat/TrigCtrlStatConfigConverter.h"
#include "device-modules/TtlIo/TtlIoConfigConverter.h"
#include "device-modules/WaveRec/WaveRecConfigConverter.h"
#include "device-modules/WaveTrig/WaveTrigConfigConverter.h"
// cores
#include "device-modules/AdcSystem/AdcSystemConfigConverter.h"
#include "device-modules/Hrb/HrbModuleConfigConverter.h"
#include "device-modules/MscCore/MscCoreConfigConverter.h"
#include "device-modules/TdcCore/TdcCoreConfigConverter.h"
#include "device-modules/TqdcCore/TqdcCoreConfigConverter.h"
#include "device-modules/TrcCore/TrcCoreConfigConverter.h"
#include "device-modules/Ttb/TtbModuleConfigConverter.h"
#include "device-modules/Ttvxs/TtvxsCoreConfigConverter.h"

#include "device-modules/Tlu16SfpTrigger/Tlu16SfpTriggerConfigConverter.h"
#include "device-modules/Tlu40LvdsLuminosity/Tlu40LvdsLuminosityConfigConverter.h"
#include "device-modules/Tlu40LvdsTofTrigger/Tlu40LvdsTofTriggerConfigConverter.h"
#include "device-modules/TluDecDes/TluDecDesConfigConverter.h"
#include "device-modules/TluTtlIoControl/TluTtlIoControlConfigConverter.h"
namespace {
constexpr const char *k_device_index = "device_index";
constexpr const char *k_enabled = "enabled";
constexpr const char *k_run = "run";
// modules
constexpr auto k_ad5622 = "ad5622";
constexpr auto k_adcErc = "adcErc";
constexpr auto k_adcLemoMux = "adcLemoOutput"; // keep old name of config section
constexpr auto k_adcMpdTrig = "adcMpdTrig";
constexpr auto k_clockControl = "clockControl";
constexpr auto k_crosspointSwitch = "crosspointSwitch";
constexpr auto k_dlnpCalibrationTrigger = "dlnpCalibrationTrigger";
constexpr auto k_feLink = "feLink";
constexpr auto k_firFilter = "firFilter";
constexpr auto k_flashProg = "flashProg";
constexpr auto k_networkPort = "networkPort";
constexpr auto k_runLogic = "runLogic";
constexpr auto k_spillControl= "spillControl";
constexpr auto k_serDesDelay= "serDesDelay";
constexpr auto k_statReadout = "statReadout";
constexpr auto k_tlu16SfpTrigger = "tlu16SfpTrigger";
constexpr auto k_tlu40lvdsLuminosity = "tlu40lvdsLuminosity";
constexpr auto k_tlu40lvdsTofTrigger = "tlu40lvdsTofTrigger";
constexpr auto k_tluDecDes = "tluDecDes";
constexpr auto k_tluTtlIo = "tluTtlIo";
constexpr auto k_trigCtrlStat = "trigCtrlStat";
constexpr auto k_ttlIo = "ttlIo";
constexpr auto k_waveBlc = "waveBlc";
constexpr auto k_periodicalAdcDelayCheck = "periodicalAdcDelayCheck";
// cores
constexpr auto k_calResult = "calResult";
constexpr auto k_adcSystemCore = "adcSystemCore";
constexpr auto k_hrb = "hrb";
constexpr auto k_msc = "msc";
constexpr auto k_tdc = "tdc";
constexpr auto k_tqdc = "tqdc";
constexpr auto k_trc = "trc";
constexpr auto k_ttb = "ttb";
constexpr auto k_ttvxs = "ttvxs";
constexpr auto k_waveRec = "waveRec";
constexpr auto k_waveTrig = "waveTrig";
} // namespace

template<>
bool AbstractConfigConverter<ModularDeviceConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<ModularDeviceConfig>::toJsonObject(const ModularDeviceConfig &in)
{
    QVariantMap m;
    // base fields
    m[k_device_index] = QVariant::fromValue(in.index);
    m[k_enabled] = QVariant::fromValue(in.enabled);
    m[k_run] = QVariant::fromValue(in.run);
    // modules
    m[k_ad5622] = QVariant::fromValue(in.ad5622);
    m[k_adcErc] = QVariant::fromValue(in.adcErc);
    m[k_adcLemoMux] = QVariant::fromValue(in.adcLemoMux);
    m[k_adcMpdTrig] = QVariant::fromValue(in.adcMpdTrig);
    m[k_clockControl] = QVariant::fromValue(in.clockControl);
    m[k_crosspointSwitch] = QVariant::fromValue(in.crosspointSwitch);
    m[k_dlnpCalibrationTrigger] = QVariant::fromValue(in.dlnpCalibrationTrigger);
    m[k_feLink] = QVariant::fromValue(in.feLink);
    m[k_firFilter] = QVariant::fromValue(in.firFilter);
    m[k_flashProg] = QVariant::fromValue(in.flashProg);
    m[k_networkPort] = QVariant::fromValue(in.networkPort);
    m[k_runLogic] = QVariant::fromValue(in.runLogic);
    m[k_spillControl] = QVariant::fromValue(in.spillControl);
    m[k_serDesDelay] = QVariant::fromValue(in.serDesDelay);
    m[k_statReadout] = QVariant::fromValue(in.statReadout);
    m[k_tlu16SfpTrigger] = QVariant::fromValue(in.tlu16SfpTrigger);
    m[k_tlu40lvdsLuminosity] = QVariant::fromValue(in.tlu40lvdsLuminosity);
    m[k_tlu40lvdsTofTrigger] = QVariant::fromValue(in.tlu40lvdsTofTrigger);
    m[k_tluDecDes] = QVariant::fromValue(in.tluDecDes);
    m[k_tluTtlIo] = QVariant::fromValue(in.tluTtlIo);
    m[k_trigCtrlStat] = QVariant::fromValue(in.trigCtrlStat);
    m[k_ttlIo] = QVariant::fromValue(in.ttlIo);
    m[k_waveBlc] = QVariant::fromValue(in.waveBlc);
    m[k_waveRec] = QVariant::fromValue(in.waveRec);
    m[k_waveTrig] = QVariant::fromValue(in.waveTrig);

    m[k_periodicalAdcDelayCheck] = QVariant::fromValue(in.periodicalAdcDelayCheck);

    // cores
    m[k_calResult] = QVariant::fromValue(in.calResult);
    m[k_adcSystemCore] = QVariant::fromValue(in.adcSystemCore);
    m[k_hrb] = QVariant::fromValue(in.hrb);
    m[k_msc] = QVariant::fromValue(in.mscCore);
    m[k_tdc] = QVariant::fromValue(in.tdcCore);
    m[k_tqdc] = QVariant::fromValue(in.tqdcCore);
    m[k_trc] = QVariant::fromValue(in.trcCore);
    m[k_ttb] = QVariant::fromValue(in.ttb);
    m[k_ttvxs] = QVariant::fromValue(in.ttvxsCore);
    return fromVariantMap(m);
}

template<>
ModularDeviceConfig AbstractConfigConverter<ModularDeviceConfig>::fromJsonObject(const QJsonObject &in)
{
    ModularDeviceConfig out;
    // base fields
    update_value(in, k_device_index, &out.index);
    update_value(in, k_enabled, &out.enabled);
    update_value(in, k_run, &out.run);
    // modules
    update_value(in, k_ad5622, &out.ad5622);
    update_value(in, k_adcErc, &out.adcErc);
    update_value(in, k_adcLemoMux, &out.adcLemoMux);
    update_value(in, k_adcMpdTrig, &out.adcMpdTrig);
    update_value(in, k_clockControl, &out.clockControl);
    update_value(in, k_crosspointSwitch, &out.crosspointSwitch);
    update_value(in, k_dlnpCalibrationTrigger, &out.dlnpCalibrationTrigger);
    update_value(in, k_feLink, &out.feLink);
    update_value(in, k_firFilter, &out.firFilter);
    update_value(in, k_flashProg, &out.flashProg);
    update_value(in, k_networkPort, &out.networkPort);
    update_value(in, k_runLogic, &out.runLogic);
    update_value(in, k_spillControl, &out.spillControl);
    update_value(in, k_serDesDelay, &out.serDesDelay);
    update_value(in, k_statReadout, &out.statReadout);
    update_value(in, k_tlu16SfpTrigger, &out.tlu16SfpTrigger);
    update_value(in, k_tlu40lvdsLuminosity, &out.tlu40lvdsLuminosity);
    update_value(in, k_tlu40lvdsTofTrigger, &out.tlu40lvdsTofTrigger);
    update_value(in, k_tluDecDes, &out.tluDecDes);
    update_value(in, k_tluTtlIo, &out.tluTtlIo);
    update_value(in, k_trigCtrlStat, &out.trigCtrlStat);
    update_value(in, k_ttlIo, &out.ttlIo);
    update_value(in, k_waveBlc, &out.waveBlc);
    update_value(in, k_waveRec, &out.waveRec);
    update_value(in, k_waveTrig, &out.waveTrig);

    update_value(in, k_periodicalAdcDelayCheck, &out.periodicalAdcDelayCheck);

    // cores
    update_value(in, k_calResult, &out.calResult);
    update_value(in, k_adcSystemCore, &out.adcSystemCore);
    update_value(in, k_hrb, &out.hrb);
    update_value(in, k_msc, &out.mscCore);
    update_value(in, k_tdc, &out.tdcCore);
    update_value(in, k_tqdc, &out.tqdcCore);
    update_value(in, k_trc, &out.trcCore);
    update_value(in, k_ttb, &out.ttb);
    update_value(in, k_ttvxs, &out.ttvxsCore);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<ModularDeviceConfig>::toJsonValue(const ModularDeviceConfig &in)
{
    return toJsonObject(in);
}

template<>
ModularDeviceConfig AbstractConfigConverter<ModularDeviceConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<ModularDeviceConfig>::registerSubTypes()
{
    // modules
    AbstractConfigConverter<AdcErcConfig>::registerTypes();
    AbstractConfigConverter<AdcLemoMuxCtrlConfig>::registerTypes();
    AbstractConfigConverter<AdcMpdTrigConfig>::registerTypes();
    AbstractConfigConverter<AdcSerDesDelayMap>::registerTypes();
    AbstractConfigConverter<ClockControlConfig>::registerTypes();
    AbstractConfigConverter<CrosspointSwitchConfig>::registerTypes();
    AbstractConfigConverter<DlnpCalibrationTriggerConfig>::registerTypes();
    AbstractConfigConverter<FeLinkConfig>::registerTypes();
    AbstractConfigConverter<FirFilterConfig>::registerTypes();
    AbstractConfigConverter<FlashProgModuleConfig>::registerTypes();
    AbstractConfigConverter<NetworkPortConfig>::registerTypes();
    AbstractConfigConverter<RunLogicConfig>::registerTypes();
    AbstractConfigConverter<SpillConfig>::registerTypes();
    AbstractConfigConverter<StatReadoutConfig>::registerTypes();
    AbstractConfigConverter<Tlu16SfpTriggerConfig>::registerTypes();
    AbstractConfigConverter<Tlu40LvdsLuminosityConfig>::registerTypes();
    AbstractConfigConverter<Tlu40LvdsTofTriggerConfig>::registerTypes();
    AbstractConfigConverter<TluDecDesConfig>::registerTypes();
    AbstractConfigConverter<TluTtlIoControlConfig>::registerTypes();
    AbstractConfigConverter<TrigCtrlStatConfig>::registerTypes();
    AbstractConfigConverter<TtlIoConfig>::registerTypes();
    AbstractConfigConverter<WaveBlcConfig>::registerTypes();
    AbstractConfigConverter<WaveRecConfig>::registerTypes();
    AbstractConfigConverter<WaveTrigConfig>::registerTypes();
    AbstractConfigConverter<Ad5622Config>::registerTypes();
    // cores
    AbstractConfigConverter<CalResult>::registerTypes();
    AbstractConfigConverter<DominoSettings>::registerTypes();
    AbstractConfigConverter<HrbModuleConfig>::registerTypes();
    AbstractConfigConverter<MscCoreConfig>::registerTypes();
    AbstractConfigConverter<TdcCoreConfig>::registerTypes();
    AbstractConfigConverter<TqdcCoreConfig>::registerTypes();
    AbstractConfigConverter<TrcCoreConfig>::registerTypes();
    AbstractConfigConverter<TtbModuleConfig>::registerTypes();
    AbstractConfigConverter<TtvxsCoreConfig>::registerTypes();
}
