#include "OneWireArrayModule.h"

#include "BMStatus.h"
#include "device-modules/OneWire/OneWire.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

enum {
    REG64_BM_BASE  = 0x380,
    REG64_BM_ID_1  = 0x0,
    REG_BM_TEMP_1  = 0x4,
    REG_BM_READY_1 = 0x5,

    REG64_BM_ID_2  = 0x8,
    REG_BM_TEMP_2  = 0xC,
    REG_BM_READY_2 = 0xD,

    REG64_BM_ID_3  = 0x10,
    REG_BM_TEMP_3  = 0x14,
    REG_BM_READY_3 = 0x15,

    REG64_BM_ID_4  = 0x18,
    REG_BM_TEMP_4  = 0x1C,
    REG_BM_READY_4 = 0x1D,

    REG_BM_PWR_STATUS = 0x3F
};

OneWireArrayModule::OneWireArrayModule(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
{
}

QList<AFI_SDB_ID> OneWireArrayModule::getSupportedSdbIds()
{
    return {};
}

void OneWireArrayModule::post_init()
{
    if (!isAvailable()) {
        setBaseAddr(REG64_BM_BASE, 0x003F);
    }
}

std::optional<BMStatus> OneWireArrayModule::readStatus()
{
    if (!isAvailable())
        return {};

    BMStatus bmStatus;

    RegOpVector r;
    constexpr const int numSensors = 2;
    std::array<quint64, numSensors> ow_id;
    std::array<quint16, numSensors> temp;
    std::array<quint16, numSensors> status;
    for (int i=0; i<numSensors; i++) {
        r.RegRead64(i*8 + REG64_BM_ID_1, &ow_id[i]);
        r.RegRead16(i*8 + REG_BM_TEMP_1, &temp[i]);
        r.RegRead16(i*8 + REG_BM_READY_1, &status[i]);
    }
    r.RegRead16(REG_BM_PWR_STATUS, &bmStatus.pwr_status);
    if (!regOpExecRebased(r))
        return {};

    for (int i=0; i<numSensors; i++) {
        BMSensor sensor;
        sensor.ow_id = ow_id[i];
        bool crcValid = OneWire::oneWireCrcValid(sensor.ow_id);
        bool sensor_valid = crcValid && OneWire::oneWireIdValid(sensor.ow_id);
        sensor.temp = OneWire::convertOneWireTemp(temp[i]);
        sensor.ready = status[i] & 0x1;
        if (sensor_valid)
            bmStatus.sensors.push_back(sensor);
    }
    return bmStatus;
}
