#ifndef SPILLSTATUS_H
#define SPILLSTATUS_H

#include <QtCore>

#include "base/TaiTime.h"

typedef union {
    uint16_t all;
    struct {
        uint16_t ttl_ok: 1;
        uint16_t wr_ok: 1;
        uint16_t reserved_2_3: 2;
        uint16_t sel: 4; // 2 bits before r.25461
        uint16_t phase: 5;
        uint16_t reserved_13_14_15: 3;
    } bit;
} reg_spill_status_t;

inline bool operator==(const reg_spill_status_t &l, const reg_spill_status_t &r)
{
    return l.all == r.all;
}

struct SpillStatusBits
{
    friend bool operator==(const SpillStatusBits &, const SpillStatusBits &) = default;
    SpillStatusBits(quint16 init = 0) {reg.all = init;}
    QString toString() const;
    reg_spill_status_t reg;
};

struct SpillStatus
{
    friend bool operator==(const SpillStatus &, const SpillStatus &) = default;
    bool isReadoutPhase() const { return status.reg.bit.phase == 1<<2;}
    bool isFlushPhase() const { return status.reg.bit.phase == 1<<4;}
    TaiTime lastSpillTai() const;

    quint32 count = 0;
    quint32 curSpillTimeMs = 0;
    SpillStatusBits status = 0;
    quint32 ttlLastPeriodMs = 0;
    quint32 wrLastPeriodMs = 0;
    quint64 lastSpillTaiRaw = 0;
};

#endif // SPILLSTATUS_H
