#ifndef TDCCORECONFIG_H
#define TDCCORECONFIG_H

#include <QMap>
#include <QMetaType>

enum TdcResIndex {
    TDC_RES_INDEX_25_UNPACKED = -2,
    TDC_RES_INDEX_25 = -1,
    TDC_RES_INDEX_100 = 0,
    TDC_RES_INDEX_200 = 1,
    TDC_RES_INDEX_400 = 2,
    TDC_RES_INDEX_800 = 3,
    TDC_RES_INDEX_1600 = 4
};

struct HptdcAdjustConfig
{
    friend bool operator==(const HptdcAdjustConfig &, const HptdcAdjustConfig &) = default;
    bool mode_rc_compression() const; // = true; // REG32_TDC_MEASURE_CTRL
    bool mode_rc() const; // = true;             // REG32_TDC_MEASURE_CTRL
    quint16 leading_resolution() const; // = 0;  // REG32_TDC_MEASURE_CTRL
    quint16 dll_mode() const; // = 2;            // REG32_TDC_MEASURE_CTRL
    quint16 dll_clock() const; // = 3;           // REG32_TDC_MEASURE_CTRL

    TdcResIndex resolution = TDC_RES_INDEX_25;
    quint8 readout_fifo_size = 7;         // REG32_TDC_READ_CTRL
    quint8 max_event_size = 9;            // REG32_TDC_READ_CTRL
    quint8 dead_time = 0;                 // REG32_TDC_READ_CTRL
    bool reject_readout_fifo_full = true; // REG32_TDC_READ_CTRL
    bool enable_error_mark = true;        // REG32_TDC_READ_CTRL
    bool enable_pair = false;             // REG32_TDC_MEASURE_CTRL
    quint16 core_clock_source = 1;        // REG32_TDC_MEASURE_CTRL
    bool enable_matching = true;
};
Q_DECLARE_METATYPE(HptdcAdjustConfig)

QDebug operator<<(QDebug dbg, const HptdcAdjustConfig &s);

class TdcCoreConfig
{
    friend class TdcCoreModule;

public:
    friend bool operator==(const TdcCoreConfig &, const TdcCoreConfig &) = default;
    void mergeLocalData(const TdcCoreConfig &r) { chTdcEn = r.chTdcEn; }

    QMap<int, bool> chTdcEn;
    quint32 latency = 8;  // in 24ns steps
    quint32 matchWin = 8; // in 24ns steps

    bool hptdcHdr = false;
    bool hptdcTrl = false;

    bool leadEdge = true;
    bool trailEdge = true;

    bool autoResolution = true;
    HptdcAdjustConfig adjustConfig;
};
Q_DECLARE_METATYPE(TdcCoreConfig)

QDebug operator<<(QDebug dbg, const TdcCoreConfig &s);

#endif // TDCCORECONFIG_H
