#include "TluTtlIoControlModule.h"

#include "DeviceModuleCommand.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    REG32_ID = 0x00, // - Identification cef86b29, 32 bits, RO static
    REG_CTRL = 0x02, // - Control, R/W // [0] - softreset
    REG_CH_NUM = 0x04, // Number of channels, RO static
    REG_CH_BASE = 0x10, // R/W, 16 bit per ch
};
}

QList<AFI_SDB_ID> TluTtlIoControlModule::getSupportedSdbIds()
{
    return {AFI_SDB_TLU_TTL_IO_Control};
}

bool TluTtlIoControlModule::writeConfig(const TluTtlIoControlConfig &config)
{
    if(!buildParams)
        return {};

    RegOpVector r;
    for(auto ch=0; ch<buildParams->chNum; ++ch) {
        r.RegWrite16(REG_CH_BASE+ch, config.ttlCfg.value(ch).all);
    }

    return regOpExecRebased(r);
}

void TluTtlIoControlModule::post_init()
{
    buildParams.reset();
    initConfig.reset();
    quint32 id;
    TluTtlIoControlBuildParams bp;

    RegOpVector r;
    r.RegRead32(REG32_ID, &id);
    r.RegRead16(REG_CH_NUM, &bp.chNum);

    if(regOpExecRebased(r) && getSupportedSdbIds().contains(static_cast<AFI_SDB_ID>(id))) {
        buildParams = bp;
        if(auto cfg=readConfig())
            initConfig = cfg;
    }
}

std::optional<TluTtlIoControlConfig> TluTtlIoControlModule::readConfig()
{
    if(!buildParams)
        return {};

    TluTtlIoControlConfig cfg;
    cfg.ttlCfg.resize(buildParams->chNum);
    RegOpVector r;
    for(auto ch=0; ch<buildParams->chNum; ++ch) {
        r.RegRead16(REG_CH_BASE+ch, &cfg.ttlCfg[ch].all);
    }

    if(regOpExecRebased(r))
        return cfg;

    return {};
}
