#ifndef WAVERECMODULE_H
#define WAVERECMODULE_H

#include "device-modules/BaseDeviceModule.h"

#include "AdcBuildParams.h"
#include <optional>

class MultichannelBaselineStats;
class WaveRecConfig;
class WaveRecStatus;

class WaveRecModule : public BaseDeviceModule<WaveRecModule>
{
public:
    explicit WaveRecModule(const MregBusInterface &bus);

    static QList<AFI_SDB_ID> getSupportedSdbIds();

    void post_init() override { readBuildParams(); }

    bool writeConfig(const WaveRecConfig &config);
    bool readStatus(WaveRecStatus *result);
    std::optional<WaveRecStatus> readStatus();
    QMap<int, QVector<int> > getAllChannelData(int repeat = 1);
    bool readBaseline(MultichannelBaselineStats *result, int repeat = 1);
    std::optional<AdcBuildParams> getBuildParams() const { return buildParams; }

    mlink::RegOpVector setAdcChEnRegOp(quint64 mask);
    mlink::RegOpVector setModeRegOp(bool zsEn);
    mlink::RegOpVector setLat(quint16 lat);
    mlink::RegOpVector setMatchWin(quint16 matchWin);
    mlink::RegOpVector setPreWin(quint16 preWin);
    mlink::RegOpVector setAdcWin(quint16 adcWin);

private:
    bool readBuildParams();
    bool isV2() const;

    std::optional<AdcBuildParams> buildParams;
};

#endif // WAVERECMODULE_H
