#include "SpinBoxDelegate.h"

#include <QDebug>
#include <QTimer>
#include <QSpinBox>
#include <limits>

SpinBoxDelegate::SpinBoxDelegate(QObject *parent)
    : QItemDelegate(parent)
    , commitDataTimer(new QTimer(this))
{
    commitDataTimer->setInterval(1);
    commitDataTimer->setSingleShot(true);
    connect(commitDataTimer, &QTimer::timeout,
            this, &SpinBoxDelegate::flushCommitData);
}

QWidget *SpinBoxDelegate::createEditor(QWidget *parent,
                                       const QStyleOptionViewItem &/* option */,
                                       const QModelIndex &index) const
{
     QSpinBox *spinBox = new QSpinBox(parent);
     initSpinBox(spinBox);
     const auto &var = index.model()->data(index, Qt::EditRole);
     if (var.isNull())
         return {};
     bool ok;
     auto value = var.toDouble(&ok);
     if (ok) {
         spinBox->setValue(value);
     }
     connect(spinBox, QOverload<int>::of(&QSpinBox::valueChanged),
             this, &SpinBoxDelegate::dataChanged);

     return spinBox;
}

void SpinBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    QSpinBox *spinBox = static_cast<QSpinBox*>(editor);
    if (!spinBox)
        return;
    const auto &var = index.model()->data(index, Qt::EditRole);
    bool ok;
    auto value = var.toDouble(&ok);
    if (ok) {
        QSignalBlocker blocker(spinBox);
        spinBox->setValue(value);
    }
}

void SpinBoxDelegate::setModelData(QWidget *editor,
                                   QAbstractItemModel *model,
                                   const QModelIndex &index) const
{
    QSpinBox *spin_box = static_cast<QSpinBox*>(editor);
    spin_box->interpretText();
    int value = spin_box->value();
    if(model->data(index, Qt::EditRole).toInt() != value)
        model->setData(index, value, Qt::EditRole);
}

void SpinBoxDelegate::updateEditorGeometry(QWidget *editor,
                                           const QStyleOptionViewItem &option,
                                           const QModelIndex &/* index */) const
{
    editor->setGeometry(option.rect);
}



void AnlgThrSpinBoxDelegate::initAnlgSpinBox(QSpinBox *s)
{
    QSignalBlocker blocker(s);
    s->setSuffix(" mV");
    s->setMinimum(-3000);
    s->setMaximum(3000);
}

void SpinBoxDelegate::dataChanged()
{
    if(commitDataWidget) {
        if(commitDataWidget == sender())
            return;
        flushCommitData();
    }
    commitDataWidget = sender();
    commitDataTimer->start();
}

void SpinBoxDelegate::flushCommitData()
{
    if(!commitDataWidget)
        return;
    commitData(qobject_cast<QWidget*>(commitDataWidget));
    commitDataWidget = nullptr;
}

DeviceChModelItem *SpinBoxDelegate::getItemByIndex(const QModelIndex &index) const
{
    return static_cast<DeviceChModelItem*>(index.internalPointer());
}

void DigThrSpinBoxDelegate::initSpinBox(QSpinBox *s) const
{
    QSignalBlocker blocker(s);
    s->setMinimum(std::numeric_limits<qint16>::min());
    s->setMaximum(std::numeric_limits<qint16>::max());
}
