#ifndef FELINKTREEMODEL_H
#define FELINKTREEMODEL_H

#include <QAbstractItemModel>
#include <optional>

#include "FeLinkTreeModelItem.h"
#include "FeLinkTreeColumns.h"

#include "base/DeviceIndex.h"
#include "fsm/FsmState.h"
#include "lib-common/BaseDeviceAppConfig.h"
#include "FeLinkStatus/FeLinkBuildParams.h"
#include "FeLinkStatus/FeLinkStatus.h"

class QTimer;
class ConfigKey;
class DeviceIndexEnabledMap;

class FeLinkTreeModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    FeLinkTreeModel(QObject *parent = nullptr);
    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override
    {
        Q_UNUSED(parent)
        return static_cast<int>(FeLinkTreeColumns::COL_TOTAL);
    }

    void setEnabled(bool en);
    int getDevRow(const DeviceIndex &di) const;

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;
     void phyReset(const DeviceIndex &dev, int ch) const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);
    void setFeLinkBp(const DeviceIndex &index, const FeLinkBuildParams &bp);

    void setDeviceList(const DeviceIndexEnabledMap &map);
    void setDeviceSet(const DeviceIndexSet &set);
    void feLinkStatusUpdated(const DeviceIndex &index, const QMap<int, FeLinkStatus> &status);
    void clearStats();
    void onDeviceConnected(const DeviceIndex &devIndex);
    void onFsmStateChange(const FsmState &state);

private slots:
    void postCheckStateChange();

private:
    void enqueueConfigSend();
    void doReset();
    void sendDataChanged(QVector<int> roles = QVector<int>());
    QVariant displayData(const QModelIndex &index) const;
    QVariant fontData(const QModelIndex &index) const;
    QVariant foregroundData(const QModelIndex &index) const;
    QVariant textAlignmentData(const QModelIndex &index) const;
    QVariant toolTipData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;

    inline FeLinkTreeColumns getColumnByIndex(const QModelIndex &index) const;
    inline FeLinkTreeModelItem *getItemByIndex(const QModelIndex &index) const;
    std::optional<FeLinkStatus> getStatus(const QModelIndex &index) const;
    std::optional<FeLinkStatus> getStatusOffset(const DeviceIndex &dev, int ch) const;

    DeviceIndexSet deviceSet;
    BaseDeviceAppConfig config;
    QMap<DeviceIndex, QMap<int, FeLinkStatus>> dev_status;
    QMap<DeviceIndex, QMap<int, FeLinkStatus>> devStatusOffset;
    QMap<DeviceIndex, FeLinkBuildParams> devBp;
    FeLinkTreeModelItem rootItem;
    QTimer *const delayedSendConfigTimer;

    bool enable = true;
    bool onFreeze = false;
};

#endif // FELINKTREEMODEL_H
