#include "HistMngrCfgConverter.h"

#include "base/DeviceIndexConverter.h"
#include "base/DaqModuleConverter.h"
#include "base/ConfigConverterUtil.h"

namespace {
constexpr const char *k_hist_id = "hist_id";
constexpr const char *k_hist_group_en = "hist_group_en";
constexpr const char *k_show_fit = "show_fit";

constexpr const char *k_daq_module = "daq_module";
constexpr const char *k_is_inl_enabled = "is_inl_enabled";
constexpr const char *k_inl_hist_code_dens = "inl_hist_code_dens";
constexpr const char *k_ampl_hist_with_max = "ampl_hist_with_max";
constexpr const char *k_mw_min = "mw_min";
constexpr const char *k_mw_max = "mw_max";
constexpr const char *k_hitTimeEdgeLead = "hitTimeEdgeLead";
constexpr const char *k_hitTimeEdgeTrail = "hitTimeEdgeTrail";
constexpr const char *k_log_y_axis = "log_y_axis";
constexpr const char *k_global_enabled = "global_enabled";
constexpr const char *k_cur_hist_id = "cur_hist_id";
constexpr const char *k_tai_ts_window = "tai_ts_window";
constexpr const char *k_hide_zero_bin = "hide_zero_bin";

constexpr const char *k_cur_dev_index = "cur_dev_index";
constexpr const char *k_cur_ch = "cur_ch";
constexpr const char *k_curEdgeLead = "curEdgeLead";
constexpr const char *k_curEdgeTrail = "curEdgeTrail";

constexpr const char *k_use_stop_ch = "use_stop_ch";
constexpr const char *k_stop_dev_index = "stop_dev_index";
constexpr const char *k_stop_ch = "stop_ch";
constexpr const char *k_stopEdgeLead = "stopEdgeLead";
constexpr const char *k_stopEdgeTrail = "stopEdgeTrail";

constexpr const char *k_hist_groups = "hist_groups";
constexpr const char *k_histExportPath = "histExportPath";
}

template<>
bool AbstractConfigConverter<HistId>::classIsObject()
{
    return false;
}

template<>
QJsonObject AbstractConfigConverter<HistId>::toJsonObject(const HistId &)
{
    Q_UNREACHABLE();
}

template<>
HistId AbstractConfigConverter<HistId>::fromJsonObject(const QJsonObject &)
{
    Q_UNREACHABLE();
}

template<>
QJsonValue AbstractConfigConverter<HistId>::toJsonValue(const HistId &in)
{
    return static_cast<int>(in);
}

template<>
HistId AbstractConfigConverter<HistId>::fromJsonValue(const QJsonValue &in)
{
    return static_cast<HistId>(in.toInt());
}

template<>
void AbstractConfigConverter<HistId>::registerSubTypes()
{
    if (QMetaType::hasRegisteredConverterFunction<QString, HistId>())
        return;
    QMetaType::registerConverter<HistId, QString>(&getStrKeyForHistId);
    QMetaType::registerConverter<QString, HistId>(&getHistIdForStrKey);
}

// ---

template<>
bool AbstractConfigConverter<HistGroupCfg>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<HistGroupCfg>::toJsonObject(const HistGroupCfg &in)
{
    QVariantMap m;
    m[k_hist_id] = QVariant::fromValue(in.hist_id);
    m[k_hist_group_en] = in.hist_group_en;
    m[k_show_fit] = in.show_fit;
    return fromVariantMap(m);
}

template<>
HistGroupCfg AbstractConfigConverter<HistGroupCfg>::fromJsonObject(const QJsonObject &in)
{
    HistGroupCfg out;

    update_value(in, k_hist_id, &out.hist_id);
    update_value(in, k_hist_group_en, &out.hist_group_en);
    update_value(in, k_show_fit, &out.show_fit);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<HistGroupCfg>::toJsonValue(const HistGroupCfg &in)
{
    return toJsonObject(in);
}

template<>
HistGroupCfg AbstractConfigConverter<HistGroupCfg>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<HistGroupCfg>::registerSubTypes()
{
    AbstractConfigConverter<HistId>::registerTypes();
}

// ---

template<>
bool AbstractConfigConverter<HistMngrCfg>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<HistMngrCfg>::toJsonObject(const HistMngrCfg &in)
{
    QVariantMap m;
    m[k_daq_module] = QVariant::fromValue(in.daq_module);
    m[k_is_inl_enabled] = QVariant::fromValue(in.is_inl_enabled);
    m[k_inl_hist_code_dens] = QVariant::fromValue(in.inl_hist_code_dens);
    m[k_ampl_hist_with_max] = QVariant::fromValue(in.ampl_hist_with_max);
    m[k_mw_min] = QVariant::fromValue(in.mw_min);
    m[k_mw_max] = QVariant::fromValue(in.mw_max);
    m[k_log_y_axis] = QVariant::fromValue(in.log_y_axis);
    m[k_global_enabled] = QVariant::fromValue(in.global_enabled);
    m[k_cur_hist_id] = QVariant::fromValue(in.cur_hist_id);
    m[k_tai_ts_window] = QVariant::fromValue(in.tai_ts_window);
    m[k_hide_zero_bin] = QVariant::fromValue(in.hide_zero_bin);

    m[k_cur_dev_index] = QVariant::fromValue(in.cur_dev_index);
    m[k_cur_ch] = QVariant::fromValue(in.cur_ch);
    m[k_curEdgeLead] = QVariant::fromValue(in.curEdgeLead);
    m[k_curEdgeTrail] = QVariant::fromValue(in.curEdgeTrail);

    m[k_use_stop_ch] = QVariant::fromValue(in.use_stop_ch);
    m[k_stop_dev_index] = QVariant::fromValue(in.stop_dev_index);
    m[k_stop_ch] = QVariant::fromValue(in.stop_ch);
    m[k_stopEdgeLead] = QVariant::fromValue(in.stopEdgeLead);
    m[k_stopEdgeTrail] = QVariant::fromValue(in.stopEdgeTrail);

    m[k_hist_groups] = ConfigConverterUtil::toJsonValue<HistId, HistGroupCfg>(in.hist_groups);

    m[k_histExportPath] = QVariant::fromValue(in.histExportPath);

    return fromVariantMap(m);
}

template<>
HistMngrCfg AbstractConfigConverter<HistMngrCfg>::fromJsonObject(const QJsonObject &in)
{
    HistMngrCfg out;
    update_value(in, k_daq_module, &out.daq_module);
    update_value(in, k_is_inl_enabled, &out.is_inl_enabled);
    update_value(in, k_inl_hist_code_dens, &out.inl_hist_code_dens);
    update_value(in, k_ampl_hist_with_max, &out.ampl_hist_with_max);
    update_value(in, k_mw_min, &out.mw_min);
    update_value(in, k_mw_max, &out.mw_max);
    update_value(in, k_hitTimeEdgeLead, &out.hitTimeEdgeLead);
    update_value(in, k_hitTimeEdgeTrail, &out.hitTimeEdgeTrail);
    update_value(in, k_log_y_axis, &out.log_y_axis);
    update_value(in, k_global_enabled, &out.global_enabled);
    update_value(in, k_cur_hist_id, &out.cur_hist_id);
    update_value(in, k_tai_ts_window, &out.tai_ts_window);
    update_value(in, k_hide_zero_bin, &out.hide_zero_bin);

    update_value(in, k_cur_dev_index, &out.cur_dev_index);
    update_value(in, k_cur_ch, &out.cur_ch);
    update_value(in, k_curEdgeLead, &out.curEdgeLead);
    update_value(in, k_curEdgeTrail, &out.curEdgeTrail);

    update_value(in, k_use_stop_ch, &out.use_stop_ch);
    update_value(in, k_stop_dev_index, &out.stop_dev_index);
    update_value(in, k_stop_ch, &out.stop_ch);
    update_value(in, k_stopEdgeLead, &out.stopEdgeLead);
    update_value(in, k_stopEdgeTrail, &out.stopEdgeTrail);

    out.hist_groups = ConfigConverterUtil::toMap<HistId, HistGroupCfg>(in[k_hist_groups]);

    update_value(in, k_histExportPath, &out.histExportPath);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<HistMngrCfg>::toJsonValue(const HistMngrCfg &in)
{
    return toJsonObject(in);
}

template<>
HistMngrCfg AbstractConfigConverter<HistMngrCfg>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<HistMngrCfg>::registerSubTypes()
{
    AbstractConfigConverter<DaqModule>::registerTypes();
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<HistGroupCfg>::registerTypes();
}
