#include "JsonTreeItem.h"

#include <QJsonArray>
#include <QJsonObject>
#include <QStringList>
#include <QVariant>

JsonTreeItem::~JsonTreeItem()
{
    qDeleteAll(mChilds);
}

JsonTreeItem* JsonTreeItem::createItem(const QJsonValue& value, JsonTreeItem* parent)
{
    JsonTreeItem *item = new JsonTreeItem(parent);
    item->setKey("root");
    item->setType(value.type());

    if ( value.isObject())
    {
        //Get all QJsonValue childs
        for (QString key : value.toObject().keys()){
            QJsonValue v = value.toObject().value(key);
            JsonTreeItem * child = createItem(v,item);
            child->setKey(key);
            item->appendChild(child);
        }
    }
    else if ( value.isArray())
    {
        //Get all QJsonValue childs
        int index = 0;
        for (QJsonValue v : value.toArray()){
            JsonTreeItem * child = createItem(v,item);
            child->setKey(QString::number(index));
            item->appendChild(child);
            ++index;
        }
    }
    else
    {
        item->setValue(value.toVariant().toString());
    }

    return item;
}

void JsonTreeItem::clear()
{
    qDeleteAll(mChilds);
    mChilds.clear();
    mValue.clear();
    mType = QJsonValue::Null;
}

void JsonTreeItem::appendChild(JsonTreeItem *item)
{
    mChilds.append(item);
}

JsonTreeItem *JsonTreeItem::child(int row)
{
    return mChilds.value(row);
}

JsonTreeItem *JsonTreeItem::parent()
{
    return mParent;
}

int JsonTreeItem::childCount() const
{
    return mChilds.count();
}

int JsonTreeItem::row() const
{
    if (mParent)
        return mParent->mChilds.indexOf(const_cast<JsonTreeItem*>(this));

    return 0;
}

void JsonTreeItem::setKey(const QString &key)
{
    mKey = key;
}

void JsonTreeItem::setValue(const QString &value)
{
    mValue = value;
}

void JsonTreeItem::setType(const QJsonValue::Type &type)
{
    mType = type;
}

QString JsonTreeItem::key() const
{
    return mKey;
}

QString JsonTreeItem::value() const
{
    return mValue;
}

QJsonValue::Type JsonTreeItem::type() const
{
    return mType;
}
