#ifndef PNPBASEMAINWINDOW_H
#define PNPBASEMAINWINDOW_H

#include "base/app/BaseMainWindow.h"
#include "fsm/FsmState.h"

class ConfigurationManager;
class LogDialog;
class ProgramDescription;

class QLabel;

class PnpBaseMainWindow : public BaseMainWindow
{
    Q_OBJECT

public:
    PnpBaseMainWindow(QWidget *parent = nullptr);
    ~PnpBaseMainWindow() override;

    QAction *getActionConfigurationManger() const { return menuItemConfigurationManger; }

signals:
    void actionConfigurationManager();
    void allowRemoteControlChanged(bool);
    void actionLog();

    //pnp server
    void changeProgramDescription(const ProgramDescription &);
    void closeProgram(const ProgramDescription &);
    void set_run_state(bool isRun);

    void switch_to_config(const ConfigIndexName &conf) const; // unused just to subscribe


private:
    QAction *menuItemAllowRemoteControl = nullptr;
    QAction *menuItemConfigurationManger = nullptr;


public slots:
    void configSelectorChanged(const ConfigIndexName &cin) override;
    virtual void onFsmStateChange(const FsmState &state);
    void setAllowRemoteControl(bool allow);

protected slots:
    virtual void delayedInit() override;

protected:
    virtual void addMenuFileActions() override;
    virtual void addMenuOptionsActions() override;
    virtual void addMenuToolsActions() override;

    ProgramDescription * const program_description;
    FsmState curFsmState = FsmState::Init;

private:
    QLabel * const labelFsmState;

public:
    ConfigurationManager * const configuration_manager;
    LogDialog *const logDialog;
};

#endif // PNPBASEMAINWINDOW_H
