/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef READOUTAPPCORE_H
#define READOUTAPPCORE_H

#include "BaseAppCore.h"
#include "ModularDeviceStatus.h"

class AsyncDeviceCommander;
class DeviceDispatcher;
class DeviceIndex;
class DeviceStatusZmqPublisher;
class EvNumChecker;
class RunLogicController;
struct BaseDeviceAppConfig;

class ReadoutAppCore : public BaseAppCore
{
    Q_OBJECT
public:
    explicit ReadoutAppCore(QObject *parent = nullptr);
    virtual ~ReadoutAppCore();

signals:
    void setStatus(const ModularDeviceStatus &r);
    void adcClkDivSync(/*const ConfigKey &calibrationKey*/) const;

public slots:
    void deviceDisconnected(const DeviceIndex &index);
    void onEvNumCheckFail(const QString &mess) const;
    void requestAdcClkDivSync() const;
    virtual void setDeviceStatus(const ModularDeviceStatus &st);

protected:
    virtual void addDevice(const DeviceIndex &index) {}
    virtual void removeDevice(const DeviceIndex &index) {}
    virtual bool isRun() const override;
    virtual BaseDeviceAppConfig getBaseAppConfig() const = 0;

private:
    void feLinkRunCheck(const ModularDeviceStatus &status);
    void adcSerDesRunCheck(const ModularDeviceStatus &st);
    void trigOnXOffRunCheck(const ModularDeviceStatus &status);

public: //protected:
    AsyncDeviceCommander * const deviceCommander;
    DeviceDispatcher * const deviceDispatcher;
    DeviceStatusZmqPublisher * const deviceStatusZmqPublisher;
    EvNumChecker * const evNumChecker;
    RunLogicController * const runLogicController;

protected:
    BaseStatusMap lastDevStatus;
    QMap<DeviceIndex, ModularDeviceStatus> feLinkStatusRunReper;
};

#endif // READOUTAPPCORE_H
