/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RunControlAppConfigConverter.h"

#include "RunControlAppConfig.h"

#include "base/ConfigConverterUtil.h"
#include "lib-common/RunControlAppConfig.h"

namespace {

// RunControlAppConfig
constexpr const char *k_devicesCfg = "devicesCfg";
constexpr const char *k_default_cfg = "default_cfg";
constexpr const char *k_runEnable = "runEnable";
constexpr const char *k_localRunNumber = "localRunNumber";
constexpr const char *k_allowRemoteControl = "allowRemoteControl";
constexpr const char *k_errOnTrigOnXOff = "errOnTrigOnXOff";
constexpr const char *k_clients = "clients";
constexpr const char *k_custom_name = "custom_name";
}

// ============ RunControlAppConfig
template<>
bool AbstractConfigConverter<RunControlAppConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RunControlAppConfig>::toJsonObject(const RunControlAppConfig &in)
{
    QVariantMap m;
    m[k_devicesCfg] = ConfigConverterUtil::toJsonValue(in.knownConfigs);
    m[k_default_cfg] = QVariant::fromValue(in.defaultSetup);
    m[k_runEnable] = QVariant::fromValue(in.pc.runEnable);
    m[k_localRunNumber] = QVariant::fromValue(in.pc.localRunNumber);
    m[k_allowRemoteControl] = QVariant::fromValue(in.allowRemoteControl);
    m[k_errOnTrigOnXOff] = QVariant::fromValue(in.errOnTrigOnXOff);
    m[k_clients] = ConfigConverterUtil::toJsonValue(in.clients);
    m[k_custom_name] = ConfigConverterUtil::toJsonValue(in.custom_name);
    return fromVariantMap(m);
}

template<>
RunControlAppConfig AbstractConfigConverter<RunControlAppConfig>::fromJsonObject(const QJsonObject &in)
{
    RunControlAppConfig out;
    out.knownConfigs = ConfigConverterUtil::toMap<DeviceIndex, ModularDeviceConfig>(in[k_devicesCfg]);
    update_value(in, k_default_cfg, &out.defaultSetup);
    update_value(in, k_runEnable, &out.pc.runEnable);
    update_value(in, k_localRunNumber, &out.pc.localRunNumber);
    update_value(in, k_allowRemoteControl, &out.allowRemoteControl);
    update_value(in, k_errOnTrigOnXOff, &out.errOnTrigOnXOff);
    auto clients = ConfigConverterUtil::toMap<ClientIndex, RcClientConfig>(in[k_clients]);
    for(auto it=clients.cbegin(); it!=clients.cend(); ++it)
        out.clients.insert(it.key(), *it);
    out.custom_name = ConfigConverterUtil::toMap<DeviceIndex, QString>(in[k_custom_name]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<RunControlAppConfig>::toJsonValue(const RunControlAppConfig &in)
{
    return toJsonObject(in);
}

template<>
RunControlAppConfig AbstractConfigConverter<RunControlAppConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RunControlAppConfig>::registerSubTypes()
{
    AbstractConfigConverter<ModularDeviceConfig>::registerTypes();
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<RcRunId>::registerTypes();
    AbstractConfigConverter<ClientIndex>::registerTypes();
}
