#ifndef RCCLIENTSTATUSMODEL_H
#define RCCLIENTSTATUSMODEL_H

#include <QObject>
#include <QMap>

#include "fsm/FsmState.h"
#include "mvc/ClientIndexTableModel.h"

#include "config/RcClientConfig.h"

class ClientIndex;
class RcClientStatusTable;
class ClientIndexEnabledMap;
struct ClientInfo;

class RcClientStatusModel : public ClientIndexTableModel
{
    Q_OBJECT
public:
    enum Columns{
        COL_TYPE, COL_INDEX,
        COL_PROTOCOL, COL_IS_VME, COL_EV_NUM_CHECK, COL_HOST,
        COL_STATUS, COL_FSM, COL_TOTAL
    };

    explicit RcClientStatusModel(QObject *parent = nullptr);
    ~RcClientStatusModel() override;

    virtual ClientIndex clientIndexOf(int row) const override;
    virtual int rowOf(const ClientIndex &sourceIndex) const override;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    bool setData(const QModelIndex& index, const QVariant& value, int role) override;

    void setExpectedFsmState(const FsmState &st);

signals:
    void rcClientsConfigChanged(const RcClientConfigMap &client_config) const;

public slots:
    void setRcClientsConfig(const RcClientConfigMap &map);
    void updateClientStatus(const ClientInfo &status);

private:
    QVariant data_DisplayRole(const ClientIndex &clIndex, int column) const;
    QVariant data_CheckStateRole(const ClientIndex &clIndex, int column) const;
    QVariant data_ToolTip(const ClientIndex &clIndex, int column) const;
    QVariant data_BackgroundRole(const ClientIndex &clIndex, int column) const;
    void invalidateRow(const ClientIndex &i, const QVector<int> roles = {});

    RcClientStatusTable * const clients;
    RcClientConfigMap config;
    FsmState expectedFsmState;
};

#endif // RCCLIENTSTATUSMODEL_H
