#include "AddPnpModel.h"

namespace {
enum class AddPnpColumns {
    COL_PROG_TYPE,
    COL_HOST_NAME,
    COL_PROG_INDEX,
    COL_TOTAL,
};
QString getProgType(const ProgramDescription &pd)
{
    auto type = pd.getDaqModule().toString();

    if(pd.getDaqModule() == DaqModule(DaqModule::Type::MStream)) {
        auto devStr = pd.options.value("dev_id_str");
        if(!devStr.isEmpty())
            type += ":" + devStr;
    }

    return type;
}
}

AddPnpModel::AddPnpModel(QObject *parent)
    : QAbstractTableModel(parent)
{
}

QVariant AddPnpModel::data(const QModelIndex &index, int role) const
{
    if(!index.isValid())
        return {};
    auto pdPtr = indexToPd(index);
    if(!pdPtr)
        return {};
    switch (role) {
    // case Qt::BackgroundRole:
    //     return backgroundData(index);
    case Qt::DisplayRole:
        return displayData(pdPtr, index.column());
    case Qt::UserRole:
        return userData(pdPtr, index.column());
    }
    return {};
}

QVariant AddPnpModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if(orientation != Qt::Horizontal)
        return {};
    if(role != Qt::DisplayRole)
        return {};
    switch (static_cast<AddPnpColumns>(section)) {
    case AddPnpColumns::COL_PROG_TYPE: return "Program type";
    case AddPnpColumns::COL_HOST_NAME: return "Host name";
    case AddPnpColumns::COL_PROG_INDEX: return "Program index";
    // case AddPnpColumns::COL_PROG_TYPE: return "Type";
    default:;
    }
    return {};
}

int AddPnpModel::rowCount(const QModelIndex &parent) const
{
    return pnpFiltred.size();
}

int AddPnpModel::columnCount(const QModelIndex &) const
{
    return static_cast<int>(AddPnpColumns::COL_TOTAL);
}

void AddPnpModel::setCache(const PnpInfoCache &cache)
{
    pnpCache = cache;
    refreshFilter();
}

void AddPnpModel::setComboBoxFiler(const AddPnpComboBoxFilter &f)
{
    if(cbFilter != f) {
        cbFilter = f;
        refreshFilter();
    }
}

void AddPnpModel::refreshFilter()
{
    auto pnpFiltredNew = filteredProgs();
    if(pnpFiltredNew.keys() != pnpFiltred.keys()) {
        beginResetModel();
        pnpFiltred = pnpFiltredNew;
        endResetModel();
    } else {
        QSet<QUuid> toApdate;
        auto newIt=pnpFiltredNew.cbegin();
        for(auto oldIt=pnpFiltred.begin();oldIt!=pnpFiltred.end();) {
            bool up = false;
            if(oldIt->getDaqModule()!=newIt->getDaqModule() ||
                oldIt->getHostName()!=newIt->getHostName() ||
                oldIt->getProgramIndex()!=newIt->getProgramIndex()) {
                toApdate << oldIt.key();
                up = true;
            }
            // *oldIt = *newIt;
            // if(up) {
            //     auto i = uuidToModeIndex(oldIt.key());
            //     emit dataChanged(i, i.siblingAtColumn(columnCount()-1));
            // }
            ++newIt;
            ++oldIt;
        }
        // Q_ASSERT(pnpFiltred == pnpFiltredNew);
        pnpFiltred = pnpFiltredNew;
        for(auto uuid : toApdate) {
            auto i = uuidToModeIndex(uuid);
            emit dataChanged(i, i.siblingAtColumn(columnCount()-1));
        }
    }
}

PnpInfoCache AddPnpModel::filteredProgs()
{
    if(!cbFilter.type && !cbFilter.host && !cbFilter.index)
        return pnpCache;

    auto ret = pnpCache;
    for(auto i=ret.cbegin(); i!=ret.cend();) {
        if(cbFilter.type && i->getDaqModule() != cbFilter.type) {
            i = ret.erase(i);
            continue;
        }

        if(cbFilter.host && i->getHostName() != cbFilter.host) {
            i = ret.erase(i);
            continue;
        }

        if(cbFilter.index && i->getProgramIndex() != cbFilter.index) {
            i = ret.erase(i);
            continue;
        }
        ++i;
    }
    return ret;
}

QVariant AddPnpModel::displayData(const ProgramDescription *pd, int col) const
{
    if(!pd)
        return {};

    switch (static_cast<AddPnpColumns>(col)) {
    case AddPnpColumns::COL_PROG_TYPE:
        return getProgType(*pd);
    case AddPnpColumns::COL_PROG_INDEX:
        return pd->getProgramIndex().toString();
    case AddPnpColumns::COL_HOST_NAME:
        return pd->hostName;
    default:;
    }
    return {};
}

QVariant AddPnpModel::userData(const ProgramDescription *pd, int col) const
{
    if(pd)
        return pd->uuid;
    return {};
}

const ProgramDescription *AddPnpModel::indexToPd(const QModelIndex &index) const
{
    int r = index.row();
    for(auto it=pnpFiltred.cbegin(); it!=pnpFiltred.cend(); ++it) {
        if(r==0)
            return &it.value();
        --r;
    }
    return {};
}

QModelIndex AddPnpModel::uuidToModeIndex(const QUuid &uuid) const
{
    int r = 0;
    for(auto it=pnpFiltred.cbegin(); it!=pnpFiltred.cend(); ++it) {
        if(it->uuid == uuid)
            return index(r,0);
        ++r;
    }
    return {};
}
