//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef QXWPLOT_H
#define QXWPLOT_H

#include <QVector>
#include <QMap>
#include "qwt_plot.h"
#include "qwt_plot_curve.h"
#include <qxw/qxw_global.h>

class QwtPlotMarker;
class QxwZoomer;

typedef QwtPlotCurve *QxwCurvePtr;
typedef QwtPlotMarker *QxwMarkerPtr;

class QXW_EXPORT QxwCurveSet: public QMap<int, QxwCurvePtr>
{
private:
    QxwCurvePtr &operator[](const int & key);
    QxwCurvePtr operator[](const int & key) const;
};
typedef QMap<int, QxwMarkerPtr> QxwMarkerSet;

class QwtPlotZoomer;
class QwtPlotPanner;
class QwtPlotGrid;

class QXW_EXPORT QxwPlot : public QwtPlot
{
    Q_OBJECT
public:
    explicit QxwPlot(QWidget *parent = 0);
    virtual ~QxwPlot();

    virtual void clear();

    void setCurveBrush(int id, const QBrush &brush);
    void setCurvePen(int id, const QPen &pen);
    void setCurveStyle(int id, QwtPlotCurve::CurveStyle style);
    void setCurveSymbol(int id, const QwtSymbol &symbol);

    void setBrush(const QBrush &brush);
    void setPen(const QPen &pen);
    void setStyle(QwtPlotCurve::CurveStyle style);
    void setSymbol(const QwtSymbol &symbol);

    void setXMarkers(const std::vector<double> &v);
    void updateLegendItems();
    void enableLegend(bool enable = true);
    void updateZoomer(QxwZoomer * _zoomer);
    //QwtPlotZoomer * getZoomer() const;

protected:
    virtual void setupZoomer();
    virtual QRectF getDataBoundRect() const;
    virtual void clearCurves();
    virtual void clearXMarkers();
    QwtPlotCurve &getCurve(int id);
    bool curveAttached(int id) const;
    bool curveExists(int id) const;
    bool markrefExists(int id) const;
    bool xmarkerExists(int id) const;


    QxwCurveSet curves;
    QxwMarkerSet xmarkers;
    QxwMarkerSet markref;
    QwtPlotGrid *grid;
    QwtPlotZoomer *zoomer;
    QwtPlotPanner *panner;

signals:

public slots:
protected slots:
    void onLegendChecked(QwtPlotItem *plotItem, bool on);
};

#endif // QXWPLOT_H
