//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#include <iostream>
#include <sstream>
#include <vector>
#include <cmath>
//#include <climits>

#include <QWidget>
#include <QColor>
#include <QString>
#include "qwt_symbol.h"
#include "qwt_plot.h"
#include "qwt_plot_grid.h"
#include "qwt_plot_curve.h"
#include "qwt_plot_marker.h"
#include "qwt_plot_zoomer.h"
#include "qwt_plot_layout.h"
#include "qwt_legend.h"

#include "qxw/WaveformPlot.h"
//#include "hptimer.h"

//const double MAXDOUBLE = std::numeric_limits<double>::max();

using namespace std;

//const int min_n_curves_threshold = 20; // keep at least this much curves




WaveformPlot::WaveformPlot(QWidget *p):
        QxwPlot(p)
{

    /*
//    (void)new QwtPlotPanner(canvas());
//    (void)new QwtPlotMagnifier(canvas());

#if QT_VERSION >= 0x040000
    using namespace Qt;
#endif
    canvas()->setFocusPolicy(WheelFocus);
*/

    //	const QwtDoubleRect &dataBoundRect = getDataBoundRect();
    //    setAxisScale(QwtPlot::yLeft, dataBoundRect.bottom(), dataBoundRect.top());
    //    enableAxis(QwtPlot::yLeft);

    setAxisAutoScale(QwtPlot::yLeft);
    setAxisAutoScale(QwtPlot::xBottom);
}

WaveformPlot::~WaveformPlot()
{
}

QColor WaveformPlot::standardColor(int index)
{
    static const QVector<QColor> map {
        {QColor(255, 150, 0)},
        {QColor(255, 0, 0)},
        {QColor(255, 0, 190)},
        {QColor(110, 0, 190)},
        {QColor(0, 100, 255)},
        {QColor(137, 137, 255)},
        {QColor(0, 200, 200)},
        {QColor(0, 190, 0)}
    };
    return map[index % map.size()];
}

void WaveformPlot::setWaveformData(const WaveformDataAll &data)
{
    bool wasAutoReplot = autoReplot();
    setAutoReplot(false);

    // curve[n] may have 3 states: NULL, hidden, shown.
    // All existing curves, shown and hidden

    //  std::cout << "WaveformPlot::setAllData, data size="<< data.size() <<std::endl;

    // make ids: count waveforms for same channel
    std::map<int, int> ids;

    // make map of new shown curves
    std::map<int, bool> shown;
    for(unsigned int i=0;i<data.size();i++) {
        const WaveformData &wd = data[i];
        if (wd.p.size() < 1)
            continue;
        int id = wd.c + ids[wd.c] * 8; // FIXME: why 8?
        ids[wd.c]++;
        shown[id] = true;
        QPen curvePen = QPen(standardColor(id), 2);
        QString curveName = wd.name;
        if (curveName.isEmpty()) {
            curveName = QString("%1").arg(wd.c);
        }
        if (!curveExists(id)) { // create new curve and attach it
            //	  std::cout << "creating new curve with id " << id << std::endl;
            // insert new curve
            QxwCurvePtr crv(new QwtPlotCurve(curveName));
            //	  crv->setSymbol (QwtSymbol (QwtSymbol::XCross, black, black, QSize (2, 2)));
            crv->setPen (curvePen);
            crv->attach(this);
            curves.insert(id, crv);
        }
        QxwCurvePtr curve = curves.value(id);
        if (!curve->plot()) { // not attached, do it
            //	  std::cout << "attaching curve with id " << id << std::endl;
            curve->attach(this);
        }
        // set data
        if (1) {
            std::vector<double> x, y;
            uint len = wd.p.size();
            x.resize(len);
            y.resize(len);
            for(uint k=0;k<len;k++) {
                x[k] = wd.p[k].x;
                y[k] = wd.p[k].y;
            }
            Q_ASSERT(x.size() > 0 && y.size() > 0);
            curve->setData(&x[0], &y[0], len);
            curve->setTitle(curveName);
            //            curve->setVisible(false);
        }
        // ref mark
#ifdef ENABLE_REF_MARKERS
        if (markref.find(id) == markref.end()) {
            WFMarker mark(new QwtPlotMarker());
            mark->setLabelAlignment(Qt::AlignRight | Qt::AlignBottom);
            mark->setLineStyle(QwtPlotMarker::HLine);
            //	  mark->setLineStyle(QwtPlotMarker::NoLine);
            mark->setLinePen(curvePen);
            mark->setLabel(curveName);
            markref[id] = mark;
        }
        if (!markref[id]->plot()) { // not attached, do it
            markref[id]->attach(this);
        }
        size_t len = wd.p.size();
        double last_y = 0;
        if (len>0)
            last_y = wd.p[len-1].y;
        markref[id]->setYValue(last_y);
#endif
    }
    // now find curves, markers to hide
    foreach(int id, curves.keys()) {
        QxwCurvePtr curve = curves.value(id);
        if ( shown.find(id) == shown.end() ) {
            if (curve && curve->plot()) {
                curve->detach();
            }
            if (markrefExists(id) && markref[id]->plot()) {
                markref[id]->detach();
            }
        }
    }
    setAutoReplot(wasAutoReplot || autoReplot());

    if (zoomer->zoomRectIndex() <= 0) {
//        const QwtDoubleRect &dataBoundRect = getDataBoundRect();
//        setAxisScale(QwtPlot::yLeft, dataBoundRect.top(), dataBoundRect.bottom());
//        zoomer->setZoomBase(dataBoundRect);
        if (axisAutoScale(QwtPlot::yLeft))
            setAxisAutoScale(QwtPlot::yLeft);
        if (axisAutoScale(QwtPlot::xBottom))
            setAxisAutoScale(QwtPlot::xBottom);
        zoomer->setZoomBase();
    }

    // Enables update on new event
    for(std::map<int, bool>::const_iterator it=shown.begin(); it !=shown.end();++it){
        int key = (*it).first;
        bool value = (*it).second;
        if(value){
            setCurveSymbol(key, QwtSymbol());
            break;
        }
    }
}

void WaveformPlot::clear()
{
    clearMarkers();
    for (auto i : qAsConst(markref)) {
        if (i->plot()) {
            i->detach();
        }
    }
    QxwPlot::clear();
}

void WaveformPlot::clearMarkers()
{
    for (auto marker : qAsConst(markers)) {
        if (marker){
            if(marker->plot()) {
                marker->detach();
            }
            delete marker;
        }
    }
    markers.clear();
}

void WaveformPlot::setMarkers(const std::vector<MarkerData> &v)
{
    for (auto marker : qAsConst(markers)) {
        marker->detach();
        delete marker;
    }
    markers.clear();
    int id = 0;
    for (const auto &marker : v) {
        QxwMarkerPtr mark(new QwtPlotMarker());
        mark->setLabelAlignment(marker.align);
        mark->setLineStyle(marker.lineStyle);
        mark->setLinePen(marker.pen);
        mark->setLabel(marker.label);
        if (marker.lineStyle == QwtPlotMarker::HLine || marker.lineStyle == QwtPlotMarker::Cross)
            mark->setYValue(marker.yvalue);
        if (marker.lineStyle == QwtPlotMarker::VLine || marker.lineStyle == QwtPlotMarker::Cross)
            mark->setXValue(marker.xvalue);
        mark->attach(this);
        markers[id] = mark;
        ++id;
    }
}
